#!/bin/sh
# $Copyright$     NB: This is an auto-generated comment with copyright note below. DO NOT MODIFY OR REMOVE!
# Copyright 2009-2018 FUJITSU LIMITED
# All rights reserved
#

# NAME
#
#   srvmagt.sh - ServerView Agents Linux Install Script
#
# SYNOPSIS
#
#   [export SRVMAGT_LOG=<nonzero>]
#
#   srvmagt.sh [-p path] [-h|--help] [-f] [-n] [-v vers] [-V] \ 
#   [{install| start|trace|restart|status|diag|stop|remove}]
#
# DESCRIPTION
#
#   The srvmagt.sh shell script combines the installation of the three
#   RPM scripts of ServerView Agents Linux into one action.
#
#   It checks if the target system satifies the necessary requirements,
#   tests dependency and configuration and calls the rpm(8) command
#   that actually installs and removes the packages.
#
#   In addition it collects status information and diagnostic material
#   needed for problem analysis.
#
#   This script requires root privileges.
#
# DIAGNOSTICS
#
#   Progress messages are logged to stdout (1), error messages to stderr (2).
#   The V option controls the amount of logging.
#   The SRVMAGT_LOG variable controls logging of package scriptlets
#
# EXIT STATUS
#
#   Exit stati follow LSB convention (see /etc/init.d/skeleton)
#   0 OK                (success
#   1 Error             (generic or unspecified error
#   2 Argument Error    (invalid or excess argument(s)
#   3 Not implemented   (unimplemented feature (e.g. "reload")
#   4 Not authorized    (user had insufficient privileges
#   5 Missing tool      (program is not installed
#   6 Not configured    (program is not configured
#   7 Not running       (program is not running
#
#   150-199 reserved for application use:
#   150 Missing Kernel source (for installation via ASP)
#   151 Unsupported OS
#   152 Update failed   (kmps could not be updated)
#   153 Partially installed
#   199 SVOM installed
#
#
# SEE ALSO
#
#   srvmagt(8)

# ----------------------------------------------------------------------
# Constants
# ----------------------------------------------------------------------

MYSELF="${0##*/}"
MYVERSION="8.40-10"   # this is the version of the srvmagt package set
HASH="####################"

SRVMAGT=${SRVMAGT:="/etc/srvmagt"}
SRVMAGT_MSG=${SRVMAGT_MSG:=$SRVMAGT/messages}
SV_PLATFORM=Linux
[ -f /etc/vmware-release ] && SV_PLATFORM=VMware
CDPATH="SVSSoftware/Software/ServerView/$SV_PLATFORM"

[ "$(pwd)" = "/etc/init.d" ] && {
	echo "Don't execute $0 in the current directory /etc/init.d"
	exit 1
}

# Use LSB function library for convenience. 
# Don't redirect the command to systemctl. Otherwise status is printed out twice on SLES12 (SA 233544127).
SYSTEMD_NO_WRAP=1
[ -f /lib/lsb/init-functions ] && . /lib/lsb/init-functions

# Source ServerView function library

if [ -f ./functions ]; then
	. ./functions
else
	. /etc/srvmagt/functions
fi

# ----------------------------------------------------------------------
# General Transformations
# ----------------------------------------------------------------------

# mapStatus - map decimal exit status value to stdout
#
# $1        decimal status value
# stdout    Four separate words go to stdout:
#           {$Completed|$Aborted} decimalStatus statusWord1 statusWord2
# No return status.

function mapStatus() {
	case $1 in
	0) echo $Completed $1 $Ok ;;            # success
	1) echo $Aborted   $1 $Error ;;         # generic or unspecified error
	2) echo $Aborted   $1 $ArgumentError ;; # invalid or excess argument(s)
	3) echo $Aborted   $1 $NotImplemented ;;# unimplemented feature (e.g. "reload")
	4) echo $Aborted   $1 $NotAuthorized ;; # user had insufficient privileges
	5) echo $Aborted   $1 $MissingResource ;; # program is not installed
	6) echo $Aborted   $1 $NotConfigured ;; # program is not configured
	7) echo $Completed $1 $NotRunning ;;    # program is not running
	153) echo $Completed $1 $PartInstalled ;; # packages are partially installed
	*) echo $Aborted   $1 $Undefined ;;
	esac
}

# ----------------------------------------------------------------------
# Monolog: (Unconfirmed) output with logging
# ----------------------------------------------------------------------

function installLog() {
	DD=`date +"%b %u %T"`
	printf "$DD $HOSTNAME srvmagt.sh: $@" >> /var/log/srvmagt/log.install
}

# errorExit - exit the script because of a logical error
#
# $1        function involved (or "main")
# $2 + $3   error cause and parameter
#
# printf  - display a FORMATTED message (this is the bash built-in cmd)
#
# display - display AND log a FORMATTED message
#
# verbose - display only if -V option AND log a FORMATTED message
#
# logOnly - only log a FORMATTED message
#
# showMsg - output a FORMATTED message to various destinations
#
#
# $1        "ALWAYS" or "VERBOSE": mode in which to display this message
# $2ff      format [arguments] (see printf)
# stdout    the formatted message to /var/log/messages
#
# No return status.

function errorExit() {
	showMsg ALWAYS "SCRIPT ERROR in %s: %s '%s'! Exit.\n" "$1" "$2" "$3"
	exit 1
}

function display() {
	showMsg ALWAYS "$@"     # MUST retain individual arguments!
}

function verbose() {
	showMsg VERBOSE "$@"    # MUST retain individual arguments!
}

function logOnly() {
	showMsg SYSLOG "$@" # MUST retain individual arguments!
}

function showMsg() {
	[ $# -eq 0 ] && errorExit $FUNCNAME "$EmptyMessage" ""

	case $1 in
	ALWAYS)     # print message to syslog and stdout
		shift
		printf "$@" | logger -t "srvmagt.sh"
		isXenServer && (installLog "$@")
		printf "$@"
		;;

	VERBOSE)    # print message to syslog and stdout if -v option
		shift
		[ $Voption -eq 1 ] && {
			printf "$@" | logger -t "srvmagt.sh"
			isXenServer && (installLog "$@")
			printf "$@"
		}
		;;

	SYSLOG)     # print message only to syslog
		shift
		# logger -t "srvmagt.sh" "$@"
		printf "$@" | logger -t "srvmagt.sh"
		isXenServer && (installLog "$@")
		;;

	*) errorExit $FUNCNAME "$InvalidArgument" "$1" ;;
	esac
}

# ----------------------------------------------------------------------
# OS
# ----------------------------------------------------------------------

# getOSProperties - get the OS properties into globals OS_*
#
# No arguments.
# Returns 0 in case of success or in case of unsupported OS with fOption.

function getOSProperties() {

	SELinux="$Unknown"
	[ -f /etc/selinux/config ] && {
		type -f getenforce >/dev/null 2>&1
		if [ $? -eq 0 ]; then
			SELinux=$(getenforce)
		else
			SELinux=$(grep '^[[:space:]]*SELINUX=' /etc/selinux/config) || SELinux="$Unknown"
			SELinux=${SELinux##*=}
			SELinux=${SELinux%% *}
		fi
	}

	EDAC="$Unknown"
	[ -d /lib/modules/$(uname -r)/kernel/drivers/edac ] && {
		EDAC="disabled"
		lsmod | grep -q "edac" && EDAC="enabled"
	}

	SetSvEnv && return 0
	rc=$?
	
	[ $action != "install" -a $action != "build" ] && return 0

	# Print the disclaimer for installation on unsupported OS
	display "$InstallOnOS" "$MYNAME" "$SV_OS" "${SV_RELEASE:1}"
	display "$UnsupportedOS" "$MYNAME" "$SV_OS" "${SV_RELEASE:1}"

	[ $rc -eq 1 ] && {
		# Main version ok 
		display "$MainVersionSupported"
		return 0
	}

	[  $fOption -eq 0 ] && {
		# display "$Terminated "$MYNAME"
		[ $JapLoc -eq 0 ] &&
			display "$ForceThisInstallation"
		return 1
	}

	display "$OverruledOSvalidation"
	display "$UnsupportedOSDisclaimer" "$MYNAME"
	return 0
}

# onReferenceSystem - test if local system is a reference system
#
# No arguments.
# Returns 0 or 1 if this IS or IS NOT a reference system

_IsReference=       # 0: IS NOT a reference system; 1: IS a reference system

function onReferenceSystem() {
	local thePkgs="$Unknown" pkg name pkgName pkgArch

	# Use a naive cache and complement it to meet return status conventions

	[ -n "$_IsReference" ] && return $((! $_IsReference))

	# A "reference" system is either a VM or has appropriate tools and kernel
	# source to build ServerView modules "srvmagt-mods_src". Otherwise it is a
	# "target" system and needs "srvmsgt-mods_bin".
	#
	# NOTE: For a VM and ESX4 where no SV modules operate this is actually irrelevant
	#       but retained to avoid RPM contents changes ("functions" come with modules).

	name=$(isVM) 
	[ $? -eq 0 -o "$SV_NICKNAME" = "ESX4" -o "${SV_NICKNAME:0:2}" = "XS" -o "${SV_NICKNAME:0:1}" = "O" ] && _IsReference=1 && return $((! $_IsReference))

	# 1. Verify that a kernel source RPM exists on the system for the running kernel
	#    and that it matches its release AND architecture.

	_IsReference=0
	pkg="$NotInstalled"

	dirNameArch=$(checkKernelSource $SV_NICKNAME $SV_ARCH) && _IsReference=1 && pkg="$Exists"
	PnameArch=${dirNameArch#*:}
	pkgName=${PnameArch%:*}
	pkgArch=${PnameArch#*:}
	verbose "$MatchingKernelSource" ${pkgName:-""} ${pkgArch:-""} ${SV_RELEASE:1} $SV_ARCH "$pkg"

	# 2. Verify that RPMs with tools to build kernel modules are present.

	verbose "$RPMsOnReferenceSystem"
	for name in gawk gcc binutils glibc-devel make; do
		pkg=$(rpm -q $name) || _IsReference=0
		verbose "%-12s %s\n" $name "$pkg"
	done

	name=$Target
	[ $_IsReference -eq 1 ] && name=$Reference
	verbose "$IsAxxxSystem" "$name"

	return $((! $_IsReference))
}


# ----------------------------------------------------------------------
# ServerView Linux Agents Modules
# ----------------------------------------------------------------------

# showModsStatus - print status of ServerView modules
#
# No arguments.
# Returns 0

function showModsStatus() {
	local theModule isLoaded devExists koExists s fscModules

	[ -f /etc/srvmagt/config ] &&
		. /etc/srvmagt/config

	fscModules="smbus"

	for theModule in $fscModules; do
		s="$SVmodule $theModule"

		koExists=$DoesNotExist
	    
		case $SV_NICKNAME in
		SLES1*)
			kmpPackage=$(printf "primergy-%s-kmp-%s" "%s" ${SV_RELEASE##*-})
			rpm -q --quiet $(printf "$kmpPackage" $theModule) && koExists=$Exists ;;
		RHEL*)
			kmpPackage="kmod-%s"
			flavors=$(rpm -qf --qf '%{name}\n' /boot/System.map* | sed '/System.map/d;s/^kernel-//;s/^kernel$/NONE/' | sort -u)
			for f in $flavors; do
				[[ "${SV_RELEASE}" == *$f ]] && {
					kmpPackage="kmod-%s-$f"
					break;
				}
			done    
			rpm -q --quiet $(printf "$kmpPackage" $theModule) && koExists=$Exists ;;
		esac

		[ $koExists = $DoesNotExist ] &&
			[ -f /lib/modules/${SV_RELEASE:1}/srvmagt/$theModule.* ] && koExists=$Exists
            
		devExists=$DoesNotExist
		[ -c /dev/pci/$theModule ] && devExists=$Exists

		isLoaded=$IsNotLoaded
		while read line; do
			[ "${line%% *}" = $theModule ] && isLoaded=$IsLoaded && break
		done < /proc/modules

		printf "%27s: %s, %s, %s %s\n" "$s" "$koExists" "$isLoaded" /dev/pci/$theModule "$devExists"
	done

	devExists=$DoesNotExist
	[ -c /dev/ipmi0 ] && devExists=$Exists

	isLoaded=$IsNotLoaded
	while read line; do
		[ "${line%%_*}" = "ipmi" -o "$SV_NICKNAME" = "ESX4" ] && isLoaded=$IsLoaded && break
	done < /proc/modules

	printf "%27s: %s, %s, %s %s\n" "$NativeIPMI" "$IsConfigured" "$isLoaded" /dev/ipmi0 "$devExists"

	return 0
}

# ----------------------------------------------------------------------
# System's SNMP service
# ----------------------------------------------------------------------

# showSnmpdStatus - print status of snmpd
#
# Assume that snmpd is running, if
#   * snmpd program and process exists
#   * UDP/TCP port 161 (SNMP) is listening.
# Returns 0 or 1 if SNMP port IS or IS NOT running and listening

function showSnmpdStatus() {
	local pstat="NOT OK" agtx="NOT OK" pid port="161" snmpCommStat="localhost $Unknown"

	[ -x /usr/sbin/snmpd ] || {
		printf "$NoSuchProgram" "$SNMPdaemon"
		return 1
	}

	pid=$(mypidof /usr/sbin/snmpd) || {
		printf "$NoSuchProcess" "$SNMPdaemon"
		return 1
	}

	# See if port 161 and AgentX socket are listened to by the same pid
	LC_ALL=C $NETSTAT -nlup | grep $pid | grep -q $port && pstat="OK"
	
	$NETSTAT -nlxp | fgrep $pid | fgrep -q agentx && agtx="OK"

    
	# get snmpd communication status of localhost 
	[ -x /usr/bin/snmpget ] && \
		ping -q -c 1 localhost >/dev/null 2>&1 && \
		snmpCommStat="NOT OK" && \
		snmpget -v1 -c $(getcommunity) localhost 1.3.6.1.2.1.1.5.0 >/dev/null 2>&1 && \
		snmpCommStat="OK"

	printf "$RunningSnmpd" "$SNMPdaemon" $pid "$pstat" "$agtx" "$snmpCommStat"
	return 0
}

# testPort - test if the given TCP/UDP port is listened to by given PID
#
# $1    PID of process
# $2    Port number of process
# Returns 0 or 1 if TCP/UDP port  IS or IS NOT listened to by $1
#
# Proto Recv-Q Send-Q Local Address               Foreign Address             State       PID/Program name
# udp        0      0 0.0.0.0:32768               0.0.0.0:*                               4927/avahi-daemon
# udp        0      0 0.0.0.0:161                 0.0.0.0:*                               4176/snmpd
# tcp        0      0 0.0.0.0:3173                0.0.0.0:*                   LISTEN      7840/amDaemon
# tcp        0      0 0.0.0.0:3172                0.0.0.0:*                   LISTEN      8853/SVRemoteConnector



function testPort() {
local proto recvQ sendQ localAddr foreignAddr state pidProgram nstatport nstatpid port pid tmpid

	pid=$1
	port=$2
	while read proto recvQ sendQ localAddr foreignAddr state pidProgram; do
		nstatport=${localAddr##*:}
		[ "$nstatport" = "$port" ] && {
			nstatpid=${pidProgram%%/*}
			if [ -n "$nstatpid" ]; then
				# TCP protokol
				for tmpid in $pid; do
					[ "$nstatpid" = "$tmpid" -a "$state" = "LISTEN" ] && return 0 
				done
			else
				# UDP protokol
				nstatpid=${state%%/*}
				for tmpid in $pid; do
					[ "$nstatpid" = "$tmpid" ] && return 0 
				done
			fi
			return 1
		}
	done
	return 1
}

# testAgentX - test if AgentX socket is listened to by given PID
#
# $1    PID of process
# Returns 0 or 1 if AgentX socket IS or IS NOT listened to by $1

function testAgentX() {

	awk -v PID=$1 '
		# unix 2 [ ACC ] STREAM LISTENING 3399511 1074/snmpd /var/agentx/master
		# unix 2 [ ACC ] STREAM LISTENING 3399511 1074/snmpd /var/run/agentx/master <<<< SLES 10

		$NF == "/var/agentx/master" || $NF == "/var/run/agentx/master" {
			split ($(NF - 1), arr, "/");
			if (arr[1] == PID)
				exit 0;
			else
				exit 1;
		}' && return 0
	return 1
}

# ----------------------------------------------------------------------
# ServerView eecd, SCS and RAID
# ----------------------------------------------------------------------

# showEecdStatus - show eecd status
#
# No arguments.
# Returns 0

function showActionScript() {
	local pid

	[ -x /etc/srvmagt/srvmagtAction ] || {
		printf "$NoSuchProgram" "$SVaction"
		return 1
	}

	pid=$(ps -ef |fgrep srvmagtAction|fgrep -v grep | awk '{ print $2 }') || {
		printf "$NoSuchProcess" "$SVaction"
		return 1
	}

	printf "$Running" "$SVaction" "$pid"

	return 0
}

function showEecdStatus() {
	local pid sock="NOT OK"

	[ -x /usr/sbin/eecd ] || {
		printf "$NoSuchProgram" "$SVeecd"
		return 1
	}

	pid=$(mypidof /usr/sbin/eecd) || {
		printf "$NoSuchProcess" "$SVeecd"
		return 1
	}

	# See if eecd sockets "/dev/eecda", "/dev/eecdaj", "/dev/eecds" and "/dev/eecdsj" are listened to by eecd.

	eecdSockets=$($NETSTAT -nlxp | grep $pid | grep "/dev/eecd" | wc -l)
	[ "$eecdSockets" -eq 4 ] && sock="OK"

	printf "$RunningEecd" "$SVeecd" "$pid" "$sock"

	return 0
}

# testEecd - test if eecd sockets are listened to by given PID
#
# $1    PID of eecd process
# Returns 0 or 1 if BOTH sockets ARE or ARE NOT listened to by $1

function testEecd() {

	awk -v PID=$1 '
		BEGIN { matches = 0 }

		# unix 2 [ ACC ] STREAM LISTENING 3399511 15728/eecd {/dev/eecda|/dev/eecds}
		$NF == "/dev/eecda" || $NF == "/dev/eecds" {
			split ($(NF - 1), arr, "/")
			if (arr[1] == PID)
				matches++;
		}

		END { exit (matches) }'
	[ $? -eq 2 ] && return 0
	return 1
}

# showSCSstatus - show SVRemoteConnector status
#
# No arguments.
# Returns 0.

function showSCSstatus() {
	local pid pstat="NOT OK" port="3172"

	[ -x $SCSInstPath/SVRemoteConnector ] || {
		printf "$NoSuchProgram" "$SVremCon"
		return 1
	}

	pid=$(mypidof $SCSInstPath/SVRemoteConnector) || {
		printf "$NoSuchProcess" "$SVremCon"
		return 1
	}

	# See if SVRemoteConnector listens on TCP #3172
	# NOTE: Need LC_ALL=C for "LISTEN"

	LC_ALL=C $NETSTAT -nltp | fgrep $pid | fgrep LISTEN | fgrep -q $port && pstat="OK"
	printf "$RunningDaemon" "$SVremCon" "$pid" "$port" "$pstat"

	return 0
}

# showRAIDstatus - show RAID status
#
# No arguments.
# Returns 0.

function showRAIDstatus() {
	local pid pstat="NOT OK" port="3173"

	[ -x /usr/sbin/amDaemon ] || {
		printf "$NoSuchProgram" "$SVRaiddaemon"
		return 1
	}

	pid=$(mypidof /usr/sbin/amDaemon) || {
		printf "$NoSuchProcess" "$SVRaiddaemon"
		return 1
	}

	# See if SVRaid listens on TCP #3173
	# NOTE: Need LC_ALL=C for "LISTEN"

	count=3
	while [ $count -gt 0 ]; do
		for tmpid in $pid; do
			LC_ALL=C $NETSTAT -nltp | fgrep $tmpid | fgrep LISTEN | fgrep -q $port && pstat="OK" && break 2
		done
		sleep 1;
		count=$(($count - 1))
	done

	printf "$RunningDaemon" "$SVRaiddaemon" "$pid" "$port" "$pstat"

	return 0
}

# ----------------------------------------------------------------------
# ServerView SNMP (sub)agents
# ----------------------------------------------------------------------

# showAgtStatus - show subagent status
#
# No arguments.
# Returns 0

function showAgtStatus() {
	local pid theAgt theLine theStatus=0 s

SV_AGENTS=${AgentsToBeStarted:="sc2 bus hd unix ether bios secur status inv thr vv hpsim vme svupdate os vmesnmp"}
	for theAgt in $SV_AGENTS; do
		s="ServerView ${theAgt}agt"
		[ -x /usr/sbin/${theAgt}agt ] || {
			printf "$NoSuchProgram" "$s"
			continue
		}

		pid=$(mypidof /usr/sbin/${theAgt}agt) || {
			printf "$NoSuchProcess" "$s"
			continue
		}

		# Show what status each subagent holds about itself
		# from line 3 of its status file

		theStatus=0
		[ -f $SRVMAGT/agent_table/${theAgt}agt ] && {
			theLine=0
			while read theStatus; do
				theLine=$((theLine + 1))
				[ $theLine -eq 3 ] && break
			done < $SRVMAGT/agent_table/${theAgt}agt

			[ $theStatus -lt 1 -o $theStatus -gt 5 ] && theStatus=0
		}

		printf "$RunningAgt" "$s" "$pid" ${map[$theStatus]} $theStatus
	done
	return 0
}

# ----------------------------------------------------------------------
# Package And Component Status Functions
# ----------------------------------------------------------------------

# getPkgStatus - get status of ServerView Agents Linux' packages
#
# $1    path to directory containing packages
# Returns 0 or 1 if status is complete or not

function getPkgStatus() {
	local thePkg modsName qf model="$Unknown" ignore
	local ipaddr type="$Target"

	model=$(isVM) || {
		ignore=$(type -f dmidecode) && {
			model=$(dmidecode -s system-product-name)
			model=${model/*PRIM/PRIM}
			[ -z "$model" ] && model="$Unknown"
		}
	}

	qf="%-16{NAME} %{VERSION}-%{RELEASE}, %{INSTALLTIME:date}\n"

	# System and OS informations
	#        Server model: PRIMERGY RX220
	#            Hostname: SM3VMSLES91 (172.25.251.204)
	#    Operating System: SUSE LINUX Enterprise Server 9 (i586)
	#        Linux Kernel: 2.6.5-7.97-smp on i686, SELinux disabled

	# NOTE: use English language to force unique text labels. IP addr taken from 1st match!

	ipaddr=$(LC_ALL=C $IFCONFIG | awk '
		$1 == "inet" && $2 ~ /^addr:[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+/ {
			split ($2, arr, ":");
			if (arr[2] != "127.0.0.1") {
				print arr[2];
				exit (0);
			}
		}
		$1 == "inet" && $2 ~ /^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+/ {
			split ($2, arr, "/");
			if (arr[1] != "127.0.0.1") {
				print arr[1];
				exit (0);
			}
		}')
	[ -z $ipaddr ] && ipaddr="0.0.0.0"

	onReferenceSystem && type="$Reference"

	printf "$ServerModel"     "${model# }"
	printf "$HostName"        $HOSTNAME $ipaddr
	printf "$OperatingSystem" "$SV_OS [$SV_NICKNAME]"
	printf "$LinuxKernel"     "${SV_RELEASE:1}" "$SV_ARCH" "$SELinux" "$EDAC" "$type"

	[ $Voption -eq 1 ] && printf "$RPMstatus"

	# Installed Packages ###############################################
	#        ServerView modules: srvmagt-mods_src 4.30-12 Do 19 Okt 2006 16:52:44 CEST
	#           ServerView eecd: srvmagt-eecd     4.30-12 Do 19 Okt 2006 16:52:57 CEST
	# ServerView SNMP subagents: srvmagt-agents   4.30-12 Do 19 Okt 2006 16:52:59 CEST
	#              SNMP service: net-snmp         5.1-80.19 Sa 23 Apr 2005 21:48:42 CEST
	# Force a timestamp in English presentation to ensure plain ASCII chars.

	# The system type may change while a modules package is installed. So try both alternatives

	SV_MODULES="smbus"
	for module in $SV_MODULES; do
		case $SV_NICKNAME in
		SLES1*)
			kmpPackage=$(printf "primergy-%s-kmp-%s" $module ${SV_RELEASE##*-})
			thePkg=$(LC_ALL=C rpm -q --queryformat "$qf" $kmpPackage) || thePkg="$kmpPackage $NotInstalled"
			printf "$SVModuleKmpRPM" "$thePkg" ;;
		RHEL*)
			flavors=$(rpm -qf --qf '%{name}\n' /boot/System.map* | sed '/System.map/d;s/^kernel-//;s/^kernel$/NONE/' | sort -u)
			kmpPackage="kmod-$module"
			for f in $flavors; do
				[[ "${SV_RELEASE}" == *$f ]] && {
					kmpPackage="kmod-$module-$f"
					break;
				}
			done
			thePkg=$(LC_ALL=C rpm -q --queryformat "$qf" $kmpPackage) || thePkg="$kmpPackage $NotInstalled"
			printf "$SVModuleKmpRPM" "$thePkg" ;;
		esac
	done
    
	thePkg=$(LC_ALL=C rpm -q --queryformat "$qf" srvmagt-mods_src) || thePkg="$modsName $NotInstalled"
	printf "$SVModuleRPM" "$thePkg"

	thePkg=$(LC_ALL=C rpm -q --queryformat "$qf" srvmagt-eecd) || thePkg="$NotInstalled"
	printf "$SVEecdRPM"       "$thePkg"

	thePkg=$(LC_ALL=C rpm -q --queryformat "$qf" srvmagt-agents) || thePkg="$NotInstalled"
	printf "$SVAagentsRPM"    "$thePkg"

	thePkg=$(LC_ALL=C rpm -q --queryformat "$qf" srv-cimprovider) || thePkg="$NotInstalled"
	printf "$SVCimProvRPM"    "$thePkg"
	
	thePkg=$(LC_ALL=C rpm -q --queryformat "$qf" SSMWebUI) || thePkg="$NotInstalled"
	printf "$SVUIRPM"    "$thePkg"

	thePkg=$(LC_ALL=C rpm -q --queryformat "$qf" ServerViewConnectorService) || thePkg="$NotInstalled"
	printf "$SVRemoteConnectorRPM"    "$thePkg"
	
	thePkg=$(LC_ALL=C rpm -q --queryformat "$qf" ServerView_RAID) || thePkg="$NotInstalled"
	printf "$SVRaidRPM"    "$thePkg"

	thePkg=$(LC_ALL=C rpm -q --queryformat "$qf" net-snmp) || thePkg="$NotInstalled"
	printf "$SNMPserviceRPM"  "$thePkg"

	printf "\n"

	[ $Voption -eq 1 ] && printf "$ModuleStatus"

	# Kernel module status #############################################
	#    ServerView module smbus: exists, unloaded, /dev/pci/smbus missing
	showModsStatus

	printf "\n"

	[ $Voption -eq 1 ] && printf "$DaemonStatus"

	# Daemon status ####################################################
	#       SNMP Daemon (snmpd): running (2293), UDP #161 OK, /var/agentx/master OK
	#    ServerView eecd daemon: running (15728), /dev/eecd[asj] OK
	#ServerView agent statusagt: running (1728), status ok (1)

	showSnmpdStatus
	showRAIDstatus

	[ -x /usr/bin/systemctl ] && showActionScript

	showEecdStatus
	showSCSstatus
	showAgtStatus

	return 0
}

# ----------------------------------------------------------------------
# Package Installation And Removal Functions
# ----------------------------------------------------------------------

# install - install perform the different installation types
#

function install() {
	local status instType="install"
	
	if [ "$SV_VENDOR" = "XenServer" ];then
		configure_iptables
		# install package net-snmp-utils, if not installed and available
		# install_netsnmp_utils "$PkgDir"
		configure_netsnmp
		instSVRAID=1
	fi

	display "$Installing" $PkgVers "$PkgDir"

	# Make sure that the version to be installed has appropriate
	# packages in the given package directory

	lookupRPMfiles "$PkgDir" $PkgVers  || exit 5

	# Carry out the install action

	chooseRPMaction $PkgVers
	rpmAction=$?
	
	[ $rpmAction -gt 0 ] && {
		# check if the ServerView AgentLess Service is installed 
		svas=$(rpm -q srvmagt-svas)  && {
			display "$InstallationConflict" $svas
			return 199
		}
	}
	
	[ $rpmAction -gt 0 -a $rpmAction -lt 4 ] && {
		if [ $SV_ARCH == "x86_64" ]; then
			# check if ServerView OperationsManager < V7.02 is installed 
			svomVers=$(rpm -q --queryformat %{VERSION} ServerViewOperationsManager)  && {
				[ $svomVers \< "7.02" ] && {
					display "$InstallationConflict" "ServerViewOperationsManager-$svomVers"
					return 199
				}
			}

			# check if VIOM < V3.5 is installed 
			viomVers=$(rpm -q --queryformat %{VERSION} viom) && {
				if ( $viomVers \< "3.05" ); then
					display "$InstallationConflict" "viom-$viomVers"
					return 199
				fi
			}
		fi
	}

	[ $rpmAction -gt 0 ] && {
		# check if the required RPM packages are installed 
		checkReq || return 5 
		requirePkg ServerViewConnectorService
		[ $? -eq 0 ] && instType="fresh"
		installConnectorService $instType || display "$Installfailed" "ServerViewConnectorService"

		if [ $instSVRAID -eq 1 ]; then
			instType="install"
			requirePkg ServerView_RAID
			[ $? -eq 0 ] && instType="fresh"
			installSVRAID $instType || display "$Installfailed" "ServerView_RAID"
		else
			checkSVRAIDVersion "$SVRDMinvers" ||
					display "$TooOld" "ServerView_RAID" "$SVRDMinvers"
		fi
	}
	case $rpmAction in
	0|nothing)
		display "$RPMsAlreadyInstalled" $PkgVers
		NoStart=1
		status=0 ;;

	1|install)      # plain install if no package exists
		[ $JapLoc -eq 1 ] && {
			. ./configureSNMP

			snmpConfPath=$(checkSNMPConf)
			[ $? -eq 0 ] && {
				display "$SnmpdNotConfigured"
				configureSnmpd $snmpConfPath
			}   
		}
		installPkgs "$PkgDir" $PkgVers
		status=$? ;;

	2|repair)       # first remove existing package, then install new version
		saveConf
		removePkgs
		installPkgs "$PkgDir" $PkgVers
		status=$?
		restoreConf ;;

	3|upgrade)  # upgrade existing packages to newer version
		Force=""
		eecdVers=$(rpm -q --queryformat %{VERSION}-%{RELEASE} srvmagt-eecd)
		[ $eecdVers \< "7.30-06" ] &&
			Force="--force"
		upgradePkgs "$PkgDir" $PkgVers $Force
		status=$? ;;

	4|downgrade)    # remove, then newly install packages of older version
		saveConf
		removePkgs
		installPkgs "$PkgDir" $PkgVers
		status=$?
		restoreConf ;;

	*) errorExit $FUNCNAME "$InvalidAction" "$rpmAction" ;;
	esac

	return $status
}

# installConnectorService - Install or fresh the Remote Connector Service Package if available
#
# Returns 0 or 1 if the package was successfully installed or not

function installConnectorService() {
	local scsPath scsRpm scsPkgVar rpmType sysArch
	
	sysArch=$(uname -m|sed 's/i.86/i386/')
	# 32-bit package is installed on RHEL5 !!!
	[ "$SV_NICKNAME" = "RHEL5" -o "$SV_NICKNAME" = "OVM32" ] && sysArch=i386
	
	rpmType="-ivh"
	[ "$1" = "fresh" ] && rpmType="-Uvh"

	for scsPath in "$PkgDir" "$PkgDir/SCS"; do
		currDIR=$(pwd)
		[ -d "$scsPath" ] || continue
		cd "$scsPath"
		for scsRpm in $(ls ServerViewConnectorService*.$sysArch.rpm 2>/dev/null); do
			scsPkgVar=${scsRpm#*ServerViewConnectorService-}
			scsPkgVar=${scsPkgVar%-*}
			[ $scsPkgVar \< $SCSMinvers ] && continue
			rpm $rpmType $scsRpm
			cd "$currDIR"
			return $?
		done
		cd "$currDIR"
	done
	display "$NoPkg" "ServerViewConnectorService" "$SCSMinvers"
	return 1    
}

# installSVRAID - Install or fresh the ServerView RAID Package if available
#
# Returns 0 or 1 if the package is successful installed or not

function installSVRAID() {
	local svrPath svrRpm svrPkgVar rpmType distrib sysArch

	rpmType="-ivh"
	[ "$1" = "fresh" ] && rpmType="-Uvh"
	
	for svrPath in "$PkgDir" "$PkgDir/../ServerView_RAID"; do
		currDIR=$(pwd)
		[ -d "$svrPath" ] || continue
		cd "$svrPath"
		case $SV_SYSTEM in
			suse)
				distrib=`echo $SV_NICKNAME | tr '[:upper:]' '[:lower:]'`
				;;
			redhat)
				distrib=`echo ${SV_NICKNAME#*RH} | tr '[:upper:]' '[:lower:]'`
				;;
			ovmserver)
				distrib=el5
				;;
			oraclelinux)
				distrib=el${SV_NICKNAME#*OL}
				;;
			xenserver)
				if [  "${SV_NICKNAME:2:2}" \>  65 ]
				then
					distrib=el7
				else
					distrib=el5
				fi
				;;
			*)
				display "$NoPkg" "ServerView_RAID" "$SVRDMinvers"
				return 1
				;;
		esac
		
		sysArch=$(uname -m|sed 's/i.86/i*86/')
			
		for svrRpm in $(ls ServerView_RAID*$distrib*.$sysArch.rpm 2>/dev/null); do
			svrPkgVar=$( rpm -q --qf "%{VERSION}-%{RELEASE}" -p ${svrRpm} )
			svrPkgVar=${svrPkgVar%.*}
			[ $svrPkgVar \< $SVRDMinvers ] && continue
			rpm $rpmType $svrRpm
			cd "$currDIR"
			return $?
		done
		cd "$currDIR"
	done
	
	# No ServerView_RAID package available. Return 0 on XENServer, if it is already installed 
	[ "$SV_VENDOR" = "XenServer" ] &&
		rpm -q --quiet ServerView_RAID && return 0
		
	display "$NoPkg" "ServerView_RAID" "$SVRDMinvers"
	return 1
}

# PLDPorDUPSupported
# No arguments.
# Returns 0 or 1 if PLDPorDUP IS or IS NOT supported

_UsePLDPorDup=       # 0: IS NOT supported; 1: IS supported

function PLDPorDUPSupported() {
	local sv_modules 

	[ -n "$_UsePLDPorDup" ] && return $((! $_UsePLDPorDup))

	_UsePLDPorDup=0

	sv_modules=""
	isPRIMEQUEST || sv_modules="smbus"

	if [ "${SV_NICKNAME:0:5}" = "SLES1" -o "${SV_NICKNAME:0:4}" = "RHEL" -a "${SV_NICKNAME:4:1}" \>  4 ]; then
		_UsePLDPorDup=1
		# Do not install in a VM or ESX 4 .
		theModel=$(isVM) &&  return $((! $_UsePLDPorDup))

		kmpsInstalled=1
		for m in $sv_modules; do
			modprobe -qn $m 2>/dev/null || {
				#  no kmp packages try to install
				display "$InstallKmp" "$m"
				installKmod $m || {
					#  error no kmp packages available
					#  the usual concept of Reference and Target system applies
					display "$InstallKmpFailed" "$m"
					_UsePLDPorDup=0
					kmpsInstalled=0
					continue
				}  
				# some systems don't report an error, so check again
				modprobe -qn $m 2>/dev/null || {
					#  error no kmp packages available
					display "$InstallKmpFailed" "$m"
					_UsePLDPorDup=0
					kmpsInstalled=0
				}
				continue
			}
			# kmp package is installed, perform an update if necessary
			if [ $SV_SYSTEM = "suse" ]; then
				kmpPackage=$(printf "primergy-%s-kmp-%s" "%s" ${SV_RELEASE##*-})
				kmpVers=$(rpm -q --qf "%{VERSION}" $(printf "$kmpPackage" $m))
			else
				flavors=$(rpm -qf --qf '%{name}\n' /boot/System.map* | sed '/System.map/d;s/^kernel-//;s/^kernel$/NONE/' | sort -u)
				for f in $flavors; do
					kmpPackage="kmod-%s"
					[ $f != "NONE" -a "${SV_RELEASE}" == *$f ] &&
						kmpPackage="kmod-%s-$f"
					kmpVers=$(rpm -q --qf "%{VERSION}" $(printf "$kmpPackage" $m)) &&
						break;
				done 
			fi

			[ "$kmpVers" \< "$KMPMinVers" ] && {
				verbose "$NoPkg" "$m" "$KMPMinVers"
				# update  kmps
				installKmod $m  
				
				kmpVers=$(rpm -q --qf "%{VERSION}" $(printf "$kmpPackage" $m)) && {
					[ "$kmpVers" \< "$KMPMinVers" ] && {
						display "$NoKMP" "$m" "$KMPMinVers"
						kmpsInstalled=2
					}
				}
			}
		done
	fi
	return $((! $_UsePLDPorDup))
}

# createRepo
# Set up a repository in order to install the kmp during update via ASP
#
function createRepo() {

	[ -d  $PkgDir/$ASP_REPODIR/$SV_REPO ] && {
		case $SV_SYSTEM in
			suse)
				isofile=`ls $PkgDir/$ASP_REPODIR/$SV_REPO/srvmagt-modules-*.iso`
				zypper ar -c -t yast2 "iso:/?iso=$isofile" __asptemp$$__ && REPO_CREATED=1
				;;

			redhat)
				if [ $SV_ARCH == "x86_64" ]; then
					isofile=`ls $PkgDir/$ASP_REPODIR/$SV_REPO/srvmagt-modules-*.x86_64.iso`
					SUBDIR=$SV_ARCH
				else
					isofile=`ls $PkgDir/$ASP_REPODIR/$SV_REPO/srvmagt-modules-*86.iso`
					SUBDIR="i386"
				fi
				BASEURL="baseurl=file:///tmp/iso$$/rpms"
				[ $SV_NICKNAME != "RHEL5" ] &&
					BASEURL="$BASEURL/$SUBDIR"
				mkdir -p /tmp/iso$$
				mount -o loop $isofile /tmp/iso$$
#				yumoption="--disablerepo=\* --enablerepo=__asptemp__"
				cat >/etc/yum.repos.d/__asptemp$$__.repo <<EOF
[__asptemp__]
name=PRIMERGY kmod packages
$BASEURL
enabled=1 
gpgcheck=1
gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-fts-linux-driver-build 
EOF
				REPO_CREATED=1
				;;
		esac
	}
}

# removeRepo
#
#
function removeRepo() {

	[ -d  $PkgDir/$ASP_REPODIR/$SV_REPO ] &&
		case $SV_SYSTEM in
			suse)
				zypper rr  __asptemp$$__
				;;

			redhat)
				rm -f /etc/yum.repos.d/__asptemp$$__.repo
				umount /tmp/iso$$
				rmdir /tmp/iso$$
				;;
		esac
}


# installKmod
#
#
function installKmod() {
	MODULES="$1"
	VENDOR=primergy
	
	# generate list of kmods
	rhel_kmod_list() {
		local f
		while [ $# -gt 0 ]; do
			for f in ${flavors[@]}; do
				case $f in
					NONE)
						echo kmod-$1
						;;
					*)
						echo kmod-$1-$f
						;;
				esac
			done
			shift
		done
	}

	sles_kmod_list() {
		local f
		while [ $# -gt 0 ]; do
			for f in ${flavors[@]}; do
				echo $VENDOR-$1-kmp-${f%-base}
			done
			shift
		done
	}

	rhel_install_cmd() {
		local m cmd arch=$(uname -m)
		cmd="yum install $yumoption -y -q"
		for m in $(rhel_kmod_list "$@"); do
			cmd="$cmd $m.$arch"
		done
		echo "$cmd"
	}

	sles_install_cmd() {
		local m
		cmd="zypper -n --gpg-auto-import-keys install -l"
		for m in $(sles_kmod_list "$@"); do
			cmd="$cmd $m"
		done
		echo "$cmd"
	}

	flavors=($(rpm -qf --qf '%{name}\n' /boot/System.map* | \
		sed '/System.map/d;s/^kernel-//;s/^kernel$/NONE/' | \
		sort -u))


	case $SV_NICKNAME in
	RHEL*)
		# check primergy-dup
		rpm -q primergy-dup >/dev/null 2>&1 || yum install $yumoption -y -q --nogpgcheck primergy-dup
		cmd="$(rhel_install_cmd $MODULES)"
	;;
	SLES*)	
		cmd="$(sles_install_cmd $MODULES)"
	;;
	  esac

	eval "$cmd"
	return $?

}


# checkReq - check if the required RPM packages are installed and log
#              appropriate messages
# Verify that the necessary RPMs are installed and also started.
# All OSs supported since 4.40 have net-snmp 5.x,
# the SNMP utilities are in a 2nd package for RHEL.
# OpenSSL service is required by the ServerView Remote Connector
# and used with the Remote SCU provider. 
# libcurl is required by Update agent (only on SLES and RedHat)
#
# Returns 0 or 1 if all packages exist or not 

function checkReq() {
	SSLMinvers="1.0.1"
	i=0
	net_snmp="net-snmp"
	openssl=""
	libstdc=""
	libcurl=""

	if [ $SV_SYSTEM = "suse" ]; then
		case $SV_NICKNAME in
		SLES11)
			openssl=libopenssl0_9_8
			libcurl=libcurl4
			if [ "$SV_ARCH" = "x86_64" ]; then
				upd=$(sed -n 's/^PATCHLEVEL = //p' /etc/SuSE-release 2>/dev/null)
				[ -z $upd ] && upd=0
				if [ $upd -le 2 ]; then 
					libstdc=libstdc++46
			else
					libstdc=libstdc++6
			fi
			else
				libstdc=libstdc++33
			fi
		;;
		*)
			openssl=libopenssl1_0_0
			libstdc=libstdc++6
			libcurl=libcurl4
		;;
		esac
		
	elif [ $SV_SYSTEM != "xenserver" ]; then
		net_snmp="${net_snmp} net-snmp-utils"
		openssl=openssl
		case $SV_NICKNAME in
		RHEL5)
			# On RHEL5 eecd and subagents are 32-bit C++-based applications 
			# and need proper C++ runtime environment!
			openssl=openssl.i686
			libstdc=compat-libstdc++-33.i386
			libcurl=curl.i386
		;;
		RHEL6)
			if [ "$SV_ARCH" = "x86_64" ]; then
				libstdc=libstdc++
			else
				libstdc=compat-libstdc++-33
			fi
			libcurl=libcurl
		;;
		RHEL7)
			libstdc=libstdc++
			libcurl=libcurl
		;;
		esac
	fi

	for package in $net_snmp $openssl $libstdc $libcurl; do
		rpm --quiet -q $package || {
			missingPkg[$i]=$package
			i=$(($i +1))
		}
	done

	[ ${#missingPkg[@]} -ne 0 ] && {
		[ "${SV_NICKNAME:0:4}" = "OVM3" ] &&  [ "$SV_NICKNAME" \< "OVM33" ] && {
			# ServerView-compat-install.sh not performed 
			display "$OVMMissingRPMS" "$SV_NICKNAME"
			return 1
		}
		for pkg in "${missingPkg[@]}"; do
			display "$RequiredRPMmissing" "$pkg"
		done
		return 1
	}
	
	sslVers=$(rpm -q --queryformat %{VERSION} $openssl)
	[ $sslVers \< $SSLMinvers ] && 
		display "$TooOld" "$openssl" "$SSLMinvers"

	return 0
}


# requirePkg - make sure that the given RPM pkg is installed and log
#              appropriate messages
#
# $1    the required RPM package name
# Returns 0 or 1 if the package exists or does not exist

function requirePkg() {
	local thePkg 

	thePkg=$(rpm -q $1) || {
		display "$RequiredRPMmissing" $1
		return 1
	}
	verbose "$RequiredRPMexists" $1 "$thePkg"
	return 0
}

# lookupRPMfiles - lookup ServerView Linux agent's package files
#
# $1    path to directory containing packages
# $2    version of packages
# Returns 0 or 1 if all packages exist or not all exist

function lookupRPMfiles() {
	local mods=0 eecd=0 agts=0 rpms="$None" modsName="srvmagt-mods_bin" sysArch=$(uname -m|sed 's/i.86/i686/')

	# 32-bit packages are installed on RHEL5 !!!
	[ "$SV_NICKNAME" = "RHEL5" -o "$SV_NICKNAME" = "OVM32" ] && sysArch=i686
	
	verbose "$LookupRPMfiles" $2 "$1"

	# When installing, expect packages prepared according to current system type

	PLDPorDUPSupported || onReferenceSystem && modsName="srvmagt-mods_src"

	# Cannot lookup RPM files residing on the Web! Trust they are there.

	case $1 in
		http:*) return 0 ;;
	esac

	[ -f "$1/$modsName-$2.$sysArch.rpm" ] && mods=1
	[ -f "$1/srvmagt-eecd-$2.$sysArch.rpm" ] && eecd=1
	[ -f "$1/srvmagt-agents-$2.$sysArch.rpm" ] && agts=1

	[ $agts -eq 1 -a $eecd -eq 1 -a $mods -eq 1 ] || {
		display "$NoRPMfiles" "$sysArch" "$2" "$1"
		rpms=$(ls -1 $1/srvmagt-*-*.rpm 2> /dev/null) || rpms="$None"
		display "$RPMsAvailable" "$rpms"
		return 1
	}
	return 0
}

# chooseRPMaction - determine action for RPM package files
#
# $1    version of packages intended to install
# Output to stdout is one of {install|repair|upgrade|downgrade|nothing}
# Returns the necessary action numerically (see mapping below)

function chooseRPMaction() {
	local mods=0 eecd=0 agts=0 provs=0 ssmUI=0
	local modsVers eecdVers agentVers modsName="srvmagt-mods_bin"
	local thePkg theAction

	# _map - maps the action value to a string

	function _map() {
		case $1 in
		0)  echo "$Nothing" ;;
		1)  echo "$Install" ;;
		2)  echo "$Repair" ;;
		3)  echo "$Upgrade" ;;
		4)  echo "$Downgrade" ;;
		*) errorExit $FUNCNAME "$InvalidArgument" "$1" ;;
		esac
	}

	# Here we know that appropriate package files exist.
	# Determine what is already installed to save configuration files:
	# If NO srvmagt packages exist, assume an "initial" installation
	#   (but test for saved configuration files anyway)
	# If ALL existing srvmagt packages have an older version,
	#   assume an "upgrade" installation, and migrate configuration files
	# If ALL existing srvmagt packages have a newer version, assume a
	#   "downgrade" installation, just save configuration files
	# Otherwise this is an undefined or incomplete configuration.
	#   "Cleanup" everything while saving configuration files.

	verbose "$LookupRPMs"

	PLDPorDUPSupported || onReferenceSystem && modsName="srvmagt-mods_src"

	modsVers=$(rpm -q --queryformat %{VERSION}-%{RELEASE} $modsName)      && mods=1
	eecdVers=$(rpm -q --queryformat %{VERSION}-%{RELEASE} srvmagt-eecd)   && eecd=1
	agentVers=$(rpm -q --queryformat %{VERSION}-%{RELEASE} srvmagt-agents) && agts=1
	provVers=$(rpm -q --queryformat %{VERSION}-%{RELEASE} srv-cimprovider) && provs=1
	ssmUIVers=$(rpm -q --queryformat %{VERSION}-%{RELEASE} SSMWebUI) && ssmUI=1

	verbose "\t$modsName=$mods srvmagt-eecd=$eecd srvmagt-agents=$agts cim-provider=$provs SVSystemMonitorWebUI=$ssmUI\n"

	[ $mods -eq 0 -a $eecd -eq 0 -a $agts -eq 0 -a $provs -eq 0 -a $ssmUI -eq 0 ] && {
		theAction=1     # Plain install
		verbose "$RPMnotInstalled" "$modsName, srvmagt-eecd, srvmagt-agents srv-cimprovider SSMWebUI" $(_map $theAction)
		return $theAction
	}

	if [ $mods -eq 0 ] ; then
		theAction=2     # Repair missing modules
		verbose "$RPMnotInstalled" $modsName $(_map $theAction)
		return $theAction
	fi

	if [ $eecd -eq 0 ] ; then
		theAction=2     # repair missing eecd (and agents)
		verbose "$RPMnotInstalled" "srvmagt-eecd" $(_map $theAction)
		return $theAction
	fi

	if [ "$modsVers" != "$eecdVers" ] ; then
		theAction=2     # repair mixed modules and eecd versions
		verbose "$MixedRPMs" \
			$modsName $modsVers "srvmagt-eecd" $eecdVers $(_map $theAction)
		return $theAction
	fi

	if [ $agts -eq 0 ] ; then
		theAction=2     # repair missing agents
		verbose "$RPMnotInstalled" "srvmagt-agents" $(_map $theAction)
		return $theAction
	fi

	if [ "$eecdVers" != "$agentVers" ] ; then
		theAction=2     # repair mixed modules and agents versions
		verbose "$MixedRPMs" \
			"srvmagt-eecd" $eecdVers "srvmagt-agents" $agentVers $(_map $theAction)
	fi
	
	if [ $provs -eq 0 ] ; then
		# srv-cimprovider missing (only on redhat ge 5.8 and sles ge 11)
		[ "$InstCIMProvs" -eq 1 ] && {
			if [ "$1" \> "$agentVers" ]; then
				theAction=3     # upgrade all
			else
				# rpm -Uhv $PkgDir/srv-cimprovider-$PkgVers.x86_64.rpm
				theAction=2     # repair srv-cimprovider
			fi
			verbose "$RPMnotInstalled" "srv-cimprovider" $(_map $theAction)
			return $theAction
		}
	else 
		if [ "$provVers" != "$agentVers" ] ; then
			theAction=2     # repair mixed cim provider and agents versions
			verbose "$MixedRPMs" \
			"srv-cimprovider" $provVers "srvmagt-agents" $agentVers $(_map $theAction)
			return $theAction
		fi
	fi
	
	if [ $ssmUI -eq 0 ] ; then
		# SSMWebUI missing (only on redhat, sles or ol)
		[ "$InstSSM" -eq 1 -a $SV_SYSTEM = "suse" -o $SV_SYSTEM = "redhat" -o $SV_SYSTEM = "oraclelinux" ] && {
			if [ "$1" \> "$agentVers" ]; then
				theAction=3     # upgrade all
			else
				theAction=2     # repair SSMWebUI
			fi
			verbose "$RPMnotInstalled" "SSMWebUI" $(_map $theAction)
			return $theAction
		}
	else 
		if [ "$ssmUIVers" != "$agentVers" ] ; then
			theAction=2     # repair mixed SSMWebUI and agents versions
			verbose "$MixedRPMs" \
			"SSMWebUI" $ssmUIVers "srvmagt-agents" $agentVers $(_map $theAction)
			return $theAction
		fi
	fi

	# Here all RPMs are installed with the same versions. Do nothing|upgrade|downgrade.

	verbose "$AllRPMsExist" $agentVers

	theAction=0
	[[ "$1" > "$agentVers" ]] && theAction=3
	[[ "$1" < "$agentVers" ]] && theAction=4

	verbose "$PerformAction" $(_map $theAction) $agentVers $1
	return $theAction
}

function deconfigure_iptables() {
	(iptables -L ServerView-INPUT >/dev/null 2>&1) || return

	display "$FirewallReconfiguredPortsClosed" "161, 3172, 3173" "INPUT"
	C=1
	while [ $C -eq 1 ];do
		/sbin/iptables -D ServerView-INPUT 1 > /dev/null 2>&1 || C=0
	done
	/sbin/iptables -D INPUT -j ServerView-INPUT
	/sbin/iptables -X ServerView-INPUT
	(mv /etc/sysconfig/iptables /etc/sysconfig/iptables_sv) && (verbose "$FileSaved" "/etc/sysconfig/iptables" "/etc/sysconfig/iptables_sv")
	/sbin/iptables-save > /etc/sysconfig/iptables
}

function configure_iptables() {
	(iptables -L ServerView-INPUT >/dev/null 2>&1) && return

	display "$FirewallReconfiguredPortsOpened" "161, 3172, 3173" "INPUT"
	/sbin/iptables -N ServerView-INPUT
	/sbin/iptables -A ServerView-INPUT -p udp -m udp --dport snmp -j ACCEPT
	/sbin/iptables -A ServerView-INPUT -p tcp -m tcp --dport 3172 -j ACCEPT
	/sbin/iptables -A ServerView-INPUT -p tcp -m tcp --dport 3173 -j ACCEPT
	/sbin/iptables -I INPUT 1 -j ServerView-INPUT
	(mv /etc/sysconfig/iptables /etc/sysconfig/iptables_orig) && (verbose "$FileSaved" "/etc/sysconfig/iptables" "/etc/sysconfig/iptables_orig")

	/sbin/iptables-save > /etc/sysconfig/iptables
}

# function install_netsnmp_utils() {
#	NET_SNMP_PKG_DIR="$1/NetSnmpUtilsPkg"
#	[ -d "$NET_SNMP_PKG_DIR" ] || (return 1)
#   
#	if ( rpm -q net-snmp-libs > /dev/null 2>&1 ); then
#		NET_SNMP_version=$(rpm -q --queryformat '%{version}-%{release}\n' net-snmp-libs )
#	else
#		return 1
#	fi
#    
#	if (! rpm -q net-snmp-utils > /dev/null 2>&1 ); then
#		for rpmFile in $NET_SNMP_PKG_DIR/net-snmp-utils-*.rpm; do
#			if [ $(rpm -q --queryformat '%{version}-%{release}\n' -p $rpmFile) = $NET_SNMP_version ]; then
#				verbose "$InstallingPkgRPM" $rpmFile $NET_SNMP_PKG_DIR "..."
#				(rpm -ihv $rpmFile ) || ( verbose "$InstallingPkgRPM" $rpmFile $NET_SNMP_PKG_DIR "failed"; echo "Installation RPM " $rpmFile " failed."; return 1 )
#			fi
#		done
#
#       fi
#	fi
#}

function set_snmpd_options() {
	# Set snmpd options to avoid bulk of logging data
	# # # # # # # # # # # # # # # # # # # # # # # # #
	fgrep LS5d /etc/sysconfig/snmpd >/dev/null 2>&1
	if [ $? -eq 1 ]
	then
		fgrep "OPTIONS=\"-c /etc/snmp/snmpd.xs.conf\"" /etc/sysconfig/snmpd  >/dev/null 2>&1
		if [ $? -eq 1 ]
		then
			mv /etc/sysconfig/snmpd /etc/sysconfig/snmpd.save
			sed '/# OPTIONS=/a\
			OPTIONS="-LS5d -Lf /dev/null -p /var/run/snmpd.pid -a"' /etc/sysconfig/snmpd.save > /etc/sysconfig/snmpd
		else
			mv /etc/sysconfig/snmpd /etc/sysconfig/snmpd.save
			sed 's/OPTIONS="-c \/etc\/snmp\/snmpd.xs.conf"/OPTIONS="-c \/etc\/snmp\/snmpd.conf -LS5d -Lf \/dev\/null -p \/var\/run\/snmpd.pid -a"/' /etc/sysconfig/snmpd.save > /etc/sysconfig/snmpd
		fi
	fi
}

function configure_netsnmp() {
# only for XenServer !!!!!
		echo "Reconfigure SNMPD"
		local systemd=0
		[ -x /usr/bin/systemctl ] && systemd=1
		if [ $systemd -eq 1 ]; then
			systemctl enable snmpd.service
		else
		chkconfig --level 2345 snmpd on
		fi

		if (! grep "svSec" /etc/snmp/snmpd.conf >/dev/null 2>&1 ); then
			# save original snmp.conf file
			mv /etc/snmp/snmpd.conf /etc/snmp/snmpd.conf_orig
			verbose "$FileSaved" "/etc/snmp/snmpd.conf" "/etc/snmp/snmpd.conf_orig"
			# write new snmp.conf file
			cat > /etc/snmp/snmpd.conf << !!
#
#  
#
# This is a basic configuration that allows ServerView
# SNMP get requests. Set <subnet>/<netmask> according to
# your environment (see snmpd.conf(5)). This configuration
# does not contain any trapsink value, and it allows
# SNMP-requests from anywhere. 
#
######################################################
com2sec svSec   default  public
#com2sec svSec  <subnet>/<netmask> public
#com2sec svSec  localhost           public
#com2sec svSec  127.0.0.1           public
group   svGroup v1       svSec
view    svView  included .1.3.6.1.2.1
view    svView  included .1.3.6.1.4.1.231.2.10
view    svView  included .1.3.6.1.4.1.231.2.49.1
view    svView  excluded .1.3.6.1.2.1.4.34
view    svView  excluded .1.3.6.1.2.1.4.35
access  svGroup "" any noauth exact svView svView none
######################################################

#trapsink 192.168.1.1 public

!!
		fi
		# SNMPD will only be started if it not running. The restart is done by the agent installation script
		if [ $systemd -eq 1 ]; then
			#XenServer 7.0
			set_snmpd_options
			systemctl restart snmpd
		else
			#XenServer 6.5 and earlier
		if [ -x /etc/init.d/snmpd ]; then
			( /etc/init.d/snmpd status > /dev/null 2>&1 ) || /etc/init.d/snmpd start
		fi
	fi

}

# installPkgs - install non-existent ServerView Linux agent's packages
#
# $1    path to directory containing packages
# $2    version of packages
# Returns 0 or >0 if install was OK or FAILED

function installPkgs() {
	local mods=0 eecd=0 agts=0 modsName="srvmagt-mods_bin" modsInit rc=0
	local sysArch=$(uname -m|sed 's/i.86/i686/')
	local systemd=0

	# 32-bit packages are installed on RHEL5 !!!
	[ "$SV_NICKNAME" = "RHEL5" -o "$SV_NICKNAME" = "OVM32" ] && sysArch=i686

	[ -x /usr/bin/systemctl ] && systemd=1

	verbose "$InstallingRPMs" "$2" "$1" "..."

	# Check if SELinux is disabled
	[ "$SV_NICKNAME" = "RHEL5" ] && {
		[ ${SELinux:1:7} = "isabled" -o "$SELinux" = "$Unknown" ] ||
		display "$SELinuxIsEnabled"
	}

	# Enable (chkconfig|insserv) necessary services
	# DISCUSSION:
	#   "Cannot insserv|chkconfig snmpd". Serious system problem that needs
	#   attention. SV Agents Linux will install themselves anyway, but will
	#   not operate without snmpd after reboot.
 
	if [ $systemd -eq 1 ]; then
		systemctl enable snmpd.service || verbose "$EnableSnmpdFailed" "systemctl(8)"
	elif [ "$SV_SYSTEM" = "suse" ]; then 
		insserv /etc/init.d/snmpd || verbose "$EnableSnmpdFailed" "insserv(8)"
	else
		chkconfig snmpd on 2>/dev/null || verbose "$EnableSnmpdFailed" "chkconfig(8)"
	fi

	# Verify, that there is sufficient disk space 
	[ $JapLoc -eq 1 ] && {
		[ checkDiskSpace ] || return 5
	}

	# Now install the srvmagt packages

	PLDPorDUPSupported || onReferenceSystem && modsName="srvmagt-mods_src"

	currDIR=$(pwd)
	cd "$1"
	rpm -ihv $modsName-$2.$sysArch.rpm && mods=1
	rpm -ihv srvmagt-eecd-$2.$sysArch.rpm && eecd=1
	rpm -ihv srvmagt-agents-$2.$sysArch.rpm && agts=1
	# install srv-cimprovider
	if [ "$InstCIMProvs" -eq 1 ]; then
		rpm -ihv srv-cimprovider-$2.x86_64.rpm || {
			verbose "$Installfailed" "srv-cimprovider"
			rc=153
		}
	else
		display "$RPMcanNotBeInstalled" "srv-cimprovider" "$reason"
	fi

	# install SSMWebUI 
	[ "$InstSSM" -eq 1 ] && 
		if [ $SV_SYSTEM = "suse" -o $SV_SYSTEM = "redhat" -o $SV_SYSTEM = "oraclelinux" ]; then
			rpm -ihv SSMWebUI-$2.noarch.rpm || {
				verbose "$Installfailed" "SV System Monitor Web UI"
				rc=153
			}
		else
			verbose "$UnsupportedOS" "SV System Monitor (Web UI)" "$SV_OS" "${SV_RELEASE:1}"
		fi
		

	cd "$currDIR"

	if [ "$SV_VENDOR" = "XenServer" ];then
		# stop agents and eecd

		# modify registry files so as to reduce memory amount for logging

		# restart agents

		# restart aura
		if [ $systemd -eq 1 ]; then
			[ -e /usr/lib/systemd/system/svraid.service ] && systemctl restart svraid.service 
		else
			[ -x /etc/init.d/aurad ] && /etc/init.d/aurad restart
		fi

	fi

	[ $agts -eq 1 -a $eecd -eq 1 -a $mods -eq 1 ] || {
		verbose "$InstallingRPMs" $2 $1 "$Failed"
		verbose "\t$modsName=$mods srvmagt-eecd=$eecd srvmagt-agents=$agts\n"
		return 1
	}

	display "$InstallingRPMs" $2 $1 "$Succeeded"
	[ $rc -gt 0 ] && return $rc
	return 0
}

tmpPkgDir="/tmp/SVTargetInst"
# prepareTargetInstall - prepare for
#
# $1    Path to directory containing packages
# $2    Name of the binary package
# $3    Path to the binary package or IP-Addr/ Host of reference system
# $4    1 if binary package resides on a remote system, 0 otherwise 
# Returncode 0 or 1 if install was OK or FAILED
#   On successful execution the return string represents the package
#   directory. In case of a failure, the error message

function prepareTargetInstall {
	local dvdDir binPkg modsBinPath answer

	dvdDir=$1
	binPkg=$2
	answer=$3
	remoteHost=$4

	cd $dvdDir
	mkdir -p $tmpPkgDir

	# Link all RPM packages from dvd to temporary installation directory
	for eachPkg in *.rpm; do
		ln -sf $dvdDir/$eachPkg $tmpPkgDir
	done

	# Get the package srvmagt-mods_bin either from the local machine or
	# from the reference system via scp
	cd $tmpPkgDir

	if [ $remoteHost -eq 0 ]; then
		modsBinPath=${answer}
		[ -f $modsBinPath/$binPkg ] || {
			printf "$InvalidPkgPath" $binPkg
			return 1
		}
		ln -sf $modsBinPath/$binPkg $tmpPkgDir || {
			printf "$SymLinkFailed" $modsBinPath $binPkg
			return 1
		}
	else
		ping -c 1 $answer >/dev/null 2>&1 || {
			printf "$InvalidIPAddr" $answer
			return 1
		}
	fi
	printf $tmpPkgDir
	return 0
}

function copyBinPackage {
	local binPkg refHost

	binPkg=$1
	refHost=$2
	scp $refHost:/etc/srvmagt/rpms/$binPkg $tmpPkgDir
	return $?
}

# upgradePkgs - upgrade existing ServerView Linux agent's packages
#
# $1    path to directory containing packages
# $2    version of packages
# Returns 0 or 1 if install was OK or FAILED

function upgradePkgs() {
	local mods=0 eecd=0 agts=0 modsName="srvmagt-mods_bin" sysArch=$(uname -m|sed 's/i.86/i686/')

	# 32-bit packages are installed on RHEL5 !!!
	[ "$SV_NICKNAME" = "RHEL5" -o "$SV_NICKNAME" = "OVM32" ] && sysArch=i686

	verbose "$UpgradingRPMs" $2 $1 "..."

	PLDPorDUPSupported || onReferenceSystem && modsName="srvmagt-mods_src" 

	stopPkgs   # make sure eecd is stopped
	
	# remove SVSystemMonitor; no longer supported 
	rpm -q --quiet SVSystemMonitor && rpm -e SVSystemMonitor

	rpm -Uhv $1/$modsName-$2.$sysArch.rpm && mods=1
	rpm -Uhv $3 $1/srvmagt-eecd-$2.$sysArch.rpm && eecd=1
	rpm -Uhv $1/srvmagt-agents-$2.$sysArch.rpm && agts=1
	[ "$InstCIMProvs" -eq 1 ] && 
		rpm -Uhv $1/srv-cimprovider-$2.x86_64.rpm|| 
			verbose "$Upgradefailed" "srv-cimprovider"
	[ "$InstSSM" -eq 1 ] && 
		if [ $SV_SYSTEM = "suse" -o $SV_SYSTEM = "redhat" -o $SV_SYSTEM = "oraclelinux" ]; then
			# workaround for 233572741: remove SSMWebUI
			rpm -e SSMWebUI >/dev/null 2>&1
			rpm -ihv $1/SSMWebUI-$2.noarch.rpm || 
			verbose "$Upgradefailed" "SV System Monitor"
		fi

	[ $agts -eq 1 -a $eecd -eq 1 -a $mods -eq 1 ] || {
		verbose "$UpgradingRPMs" $2 $1 "$Failed" \
				" (mods=$mods eecd=$eecd agts=$agts)"
		return 1
	}
	verbose "$UpgradingRPMs" $2 $1 "$Succeeded"
	return 0
}

# removePkgs - remove ServerView Linux agent's packages (if any)
#
# No arguments
# Returns 0 or 1 if removal was OK or FAILED

function removePkgs() {
	local mods=0 eecd=0 agts=0 provs=0 ssmUI=0

	verbose "$RemovingRPMs" "..."

	rpm -q --quiet SSMWebUI && ssmUI=1 && rpm -e SSMWebUI && ssmUI=0
	rpm -q --quiet srv-cimprovider && provs=1 && rpm -e srv-cimprovider && provs=0
	rpm -q --quiet srvmagt-agents && agts=1 && rpm -e srvmagt-agents && agts=0
	rpm -q --quiet srvmagt-eecd && eecd=1 && rpm -e srvmagt-eecd && eecd=0
	rpm -q --quiet srvmagt-mods_src && mods=1 && rpm -e srvmagt-mods_src && mods=0
	rpm -q --quiet srvmagt-mods_bin && mods=1 && rpm -e srvmagt-mods_bin && mods=0

	if [ "$SV_VENDOR" = "XenServer" ]; then
			rpm -q --quiet ServerView_RAID && raid=1 && rpm -e ServerView_RAID && raid=0
			deconfigure_iptables
			# remove original snmp.conf file
			[ -f /etc/snmp/snmpd.conf_orig ] && (mv /etc/snmp/snmpd.conf_orig /etc/snmp/snmpd.conf) && \
			if [  -x /usr/bin/systemctl ]; then
				#XenServer 7.0
				[ -f /etc/sysconfig/snmpd.save ] && (mv /etc/sysconfig/snmpd.save /etc/sysconfig/snmpd)
				systemctl stop snmpd
			else
				#XenServer 6.5 and earlier
				/etc/init.d/snmpd stop
			fi
	fi

	[ $ssmUI -eq 0 -a $provs -eq 0 -a $agts -eq 0 -a $eecd -eq 0 -a $mods -eq 0 ] || {
		verbose "$RemovingRPMs" "$Failed" " (mods=$mods eecd=$eecd agts=$agts provs=$provs ssmUI=$ssmUI)"
		return 1
	}
	verbose "$RemovingRPMs" "$Succeeded"
	return 0
}

# ----------------------------------------------------------------------
# Package Start And Stop Functions
# ----------------------------------------------------------------------

# startPkgs - (re)starts (installed) ServerView Linux agent's packages
#
# (re)starts the installed (!) packages in correct order with appropriate
# delays to allow for proper initialization. The called init scripts deal
# with the situation of (re)starting processes (not) already started.
#
# $1    "trace" indicated to turn on trace of agents and eecd
# Returns 0 or 1 if start was OK or FAILED

function startPkgs() {
	local mods=0 eecd=0 agts=0 action status systemd=0

	action="start"
	[ "$1" = "trace" ] && {
		action=$1
	}

	[ -x /usr/bin/systemctl ] && systemd=1

	# 32-bit packages are installed on RHEL5 !!!
	[ "$SV_NICKNAME" = "RHEL5" -o "$SV_NICKNAME" = "OVM32" ] && sysArch=i686

	if [ $systemd -eq 0 ]; then
		# check init scripts only on systems with SysVinit 
		[ -x /etc/init.d/eecd_mods_src ] && mods=1
		[ -x /etc/init.d/eecd ] && eecd=1
		[ -x /etc/init.d/srvmagt ] && agts=1

		[ $agts -eq 1 -a $eecd -eq 1 -a $mods -eq 1 ] || {
			verbose "$MissingInitScript" "$Missing" \
					" (mods=$mods eecd=$eecd agts=$agts )"
			return 5
		}
	fi

	# 1. eecd_mods_{src|bin}:
	#
	# DISCUSSION:
	# Module might not yet be compiled for this kernel or not yet loaded,
	# and automatic compilation might fail because proper kernel source
	# and/or tools are missing. 
	
	# Don't check module status here, because the start script checks it in any case

	if [ $systemd -eq 1 ]; then
		systemctl start eecd_mods.service
	else
		/etc/init.d/eecd_mods_src $action 
	fi
	
	status=$?
	
	if [ $status -ne 0 ]; then
		verbose "$StartingRPM" eecd_mods "$Failed"
		return $status
	fi
	
	verbose "$StartingRPM" eecd_mods "$Succeeded"

	# 2. eecd: eecd=1 if must start

	eecd=1
	if [ -x /etc/srvmagt/StartupScripts/eecd ]; then  
		stdout=$(/etc/srvmagt/StartupScripts/eecd status)
	else
		stdout=$(/etc/init.d/eecd status)
	fi

	status=$?
	case $status in
	0)  # All daemon are already running
		verbose "$EecdRunning"
		eecd=0 ;;

	3)  # Not running
		if [ $systemd -eq 1 ]; then
			systemctl start eecd.service
		else
			/etc/init.d/eecd $action
		fi

		status=$?
		if [ $status -ne 0 ]; then
			verbose "$StartingRPM" "eecd" "$Failed!"
			return $status
		fi

		verbose "$StartingRPM" "eecd" "$Succeeded" ;;

	*)  # Unhandled status
		verbose "$UnexpectedStatus" $status "eecd"
		return 1 ;;
	esac

	# 3. srvmagt: agts=1 if must start

	agts=1
	if [ -x /etc/srvmagt/StartupScripts/srvmagt ]; then  
		stdout=$(/etc/srvmagt/StartupScripts/srvmagt status)
	else
		stdout=$(/etc/init.d/srvmagt status)
	fi

	status=$?
	case $status in
	0)  # All subagents are already running
		verbose "$SubagentsRunning"
		agts=0 ;;

	3)  # Not all subagents are running
		if [ $systemd -eq 1 ]; then
			systemctl start srvmagt.service
		else
			/etc/init.d/srvmagt $action
		fi

		status=$?
		if [ $status -ne 0 ]; then
			verbose "$StartingRPM" "$Subagents" "$Failed"
			return $status
		fi

		verbose "$StartingRPM" "Subagents" "$Succeeded" ;;

	*)  # Unhandled status
		verbose "$UnexpectedStatus" $status "$Subagents"
		return 1 ;;
	esac

	# 4. srvmagt_scs: 
	
	if [ -x /opt/fujitsu/ServerViewSuite/SCS/srvmagt_scs ]; then
		stdout=$(/opt/fujitsu/ServerViewSuite/SCS/srvmagt_scs status)
	else
		stdout=$(/etc/init.d/srvmagt_scs status)
	fi
	
	status=$?
	case $status in
	0)  # Remote SVRemoteConnector already running
		verbose "$SVRemoteConnectorRunning"
		;;
    
	3)  # Not running
		if [ $systemd -eq 1 ]; then
			systemctl start srvmagt_scs.service
		else
			/etc/init.d/srvmagt_scs $action
		fi

		status=$?
		if [ $status -ne 0 ]; then
			verbose "$StartingRPM" "SVRemoteConnector" "$Failed"
			return $status
		fi

		verbose "$StartingRPM" "SVRemoteConnector" "$Succeeded" ;;
	
	*)  # Unhandled status
		verbose "$UnexpectedStatus" $status "SVRemoteConnector"
		return 1 ;;
	esac
	
	return 0
 }

# stopPkgs - stop (installed) ServerView Linux agent's packages
#
# stops the packages in correct order with appropriate delays
# to allow for proper termination
#
# No arguments
# Returns 0 or 1 if stop was OK or FAILED

function stopPkgs() {
	local mods=0 eecd=0 agts=0 rc=0 action status systemd=0
	
	[ -x /usr/bin/systemctl ] && systemd=1

	if [ $systemd -eq 0 ]; then
		# check init scripts only on systems with SysVinit 
		[ -x /etc/init.d/eecd_mods_src ] && mods=1
		[ -x /etc/init.d/eecd ] && eecd=1
		[ -x /etc/init.d/srvmagt ] && agts=1

		[ $agts -eq 1 -a $eecd -eq 1 -a $mods -eq 1 ] || {
			display "$MissingInitScript" "$Missing" \
					" (mods=$mods eecd=$eecd agts=$agts)!"
			return 5
		}
	fi

	# 1. srvmagt

	if [ $systemd -eq 1 ]; then
		systemctl stop srvmagt.service
	else
		/etc/init.d/srvmagt stop
	fi
	status=$?
	if [ $status -eq 0 ]; then
		verbose "$StoppingRPMs" "$Subagents" "$Succeeded"
	else
		verbose "$StoppingRPMs" "$Subagents" "$Failed"
		rc=$status
	fi

	# 2. eecd

	if [ $systemd -eq 1 ]; then
		systemctl stop eecd.service
	else
		/etc/init.d/eecd stop
	fi
	status=$?
	# If the eecd process still running kill it (SA 233660632)
	pid=$(mypidof "/usr/sbin/eecd") && {
		kill -s KILL $pid
	}
	if [ $status -eq 0 ]; then
		verbose "$StoppingRPMs" "eecd" "$Succeeded"
	else
		verbose "$StoppingRPMs" "eecd" "$Failed"
		rc=$status
	fi

	# 3. eecd_mods_src or eecd_mods_bin

	if [ $systemd -eq 1 ]; then
		systemctl stop eecd_mods.service
	else
		/etc/init.d/eecd_mods_src stop
	fi
	status=$?
	if [ $status -eq 0 ]; then
		verbose "$StoppingRPMs" eecd_mods "$Succeeded"
	else
		verbose "$StoppingRPMs" eecd_mods "$Failed"
		rc=$status
	fi


	return $rc
}

# restartCIMOM - 
# restarts the CIMOM if ServerView CIM providers are installed
#
# No arguments
# Returns 0 or 1 if restart was OK or FAILED

function restartCIMOM() {
	local systemd=0
	[ -x /usr/bin/systemctl ] && systemd=1
	
	[ ! -f /usr/share/srv-cimprovider/doc/ThirdPartyLicenseReadme.txt ] && 
		return 0    # srv-cimprovider is not installed - nothing to do
	
	[ -x "/usr/sbin/sfcbd" ] && {
		# CIMOM=sfcb
		# restart cimom if it is running
		if [ $systemd -eq 1 ]; then 
			STARTSFCB="systemctl restart sfcb.service"
			[ -f  /usr/lib/systemd/system/sblim-sfcb.service ] && STARTSFCB="systemctl restart sblim-sfcb.service"
		else 
			STARTSFCB="/etc/init.d/sfcb restart"
			[ -x /etc/init.d/sblim-sfcb ] && STARTSFCB="/etc/init.d/sblim-sfcb restart"
		fi

		if (ps -C sfcbd > /dev/null) 2>&1; then
			$STARTSFCB
		fi
	}

	[ -x "/usr/sbin/cimserver" ] && {
		# CIMOM=pegasus
		if (ps -C cimserver > /dev/null) 2>&1; then
			# cimserver is running"
			cimserver -s > /dev/null
			cimserver 
		fi
	}
}

# ----------------------------------------------------------------------
# Package Configuration Functions
# ----------------------------------------------------------------------

# saveConf - save ServerView Linux agent's configuration file(s)

function saveConf() {
	# NOT YET IMPLEMENTED

	return 0
}

# restoreConf - restore ServerView Linux agent's configuration file(s)

function restoreConf() {
	# NOT YET IMPLEMENTED

	return 0
}

# ----------------------------------------------------------------------
# Check available Disk Space
# ----------------------------------------------------------------------

# checkDiskSpace - check the system configuration
#
# No arguments
# Returns 0 or 1 if there is enough space or not

function checkDiskSpace () {
	i=1
	SizeReq[1]=21800  # /usr/lib (8000) + /usr/sbin (13000) + /usr/share (800)
	SizeReq[2]=23000  # /etc
	SizeReq[3]=2000   # /opt
	SizeReq[4]=180000 # /var/log

	for fs in /usr /etc /opt /var/log; do
		free=`df -h --block-size=1K $fs|fgrep /|awk '{print $4}'`
		[ $free -lt ${SizeReq[$i]} ]  && {
			display "$InsuffDiskSpace"
			return 1
		}
		i=$((i+1))
	done
	return 0
}

# ----------------------------------------------------------------------
# Check of System Configuration
# ----------------------------------------------------------------------

# checkConfiguration - check the system configuration
#
# No arguments
# Returns 0 or 1 if configuration is OK or NOT

function checkConfiguration() {
	# DISCUSSION:
	# o check FIXED information of the snmpd configuration file (if not
	#   there) like ServerView security names etc.
	# o check variable information of the snmpd configuration file (if
	#   not there) like trapsink and permitted source addresses

	# 1. eecd_mods_{src|bin}

	# 2. eecd

	# 3. srvmagt
	#
	# a Test snmpd.conf for the FIXED basic SNMP security information
	#   com2sec svSec    127.0.0.1           public
	#   group   svGroup  v1                  svSec
	#   view    svView   included .1
	#   access  svGroup  "" any noauth exact svView none none
	#   master  agentx
	#   agentxsocket /var/agentx/master


	errorExit $FUNCNAME "$NotYetImplemented" "check"
}

# ----------------------------------------------------------------------
# System diagnostic function
# ----------------------------------------------------------------------

# collectDiagData - Collects diagnostic status material
#
# run /etc/srvmagt/PrimeCollect (if existing); take time
# result is in /etc/srvmagt/PrimeCollect_Data.zip
#    -> /var/log/PrimeCollect/PrimeCollect_Data_YYYY_MM_DD_HHMMSS.zip
#
# No arguments
# Return the PrimeCollect return codes or 7 if PrimeCollect not exists

function collectDiagData () {
	local status

	type PrimeCollect 2> /dev/null && {
		PrimeCollect
		status=$?
		return $status
	}

	return 7
}

# getSrvmagtOpts 
# Please Note: a change of the srvmagt options must also be done in script buildSrvmagtArch.sh which builds the shell archive srvmagt.sh
##
# $1        short srvmagt options 
# No return status.

function getSrvmagtOpts() {

while getopts fhnRp:v:VJq optChar; do
	case "$optChar" in
	f)  fOption=1 ;;

	h)  printf "$BANNER\n\n" "$MYNAME" $PkgVers
		printf "$USAGE"
		exit 0 ;;

	n)  NoStart=1 ;;
    
	R)  instSVRAID=1 ;;

	p)  PkgDir="$OPTARG"

		# PkgDir tells where the packages are located:
		# + a path in a (local or remote) filesystem, like "." The
		#   shorthands "cd" or "dvd" tell to locate the packages
		#   on a locally mounted ServerView Suite CD or DVD.
		# + an URL giving the directory where the packages reside.
		#   An URL starts with "<scheme>:", see RFC 2396.
		#   The rpm command can act as an FTP and/or HTTP client.
		# Default is "."

		case "$PkgDir" in
		http:*)     : ;;
		cd|dvd)     PkgDir=$(searchCDROMs) || {
						display "$CannotAccessSVSCD"
						exit 2
					}
					PkgDir=$PkgDir"/Agents" ;;
		./*|/*)     [ -d "$PkgDir" ] || {
						display "$InvalidPkgDir" "$PkgDir"
						exit 2
					} ;;
		*)          display "$InvalidPath" "$PkgDir"
					exit 2 ;;
		esac ;;

	v)  PkgVers=$OPTARG ;;

	V)  Voption=1
		export SRVMAGT_LOG=1 ;; # turn on RPM package scriptlet logging

	J)  export JapLoc=1 ;;
	
	q)  Quiet=1 ;;
  
   	[?])
		printf "$USAGE"
		exit 2 ;;
	esac
done

shift $(($OPTIND - 1))
action=$1
[ -z "$1" ] && action=status
}


# ----------------------------------------------------------------------
# Main
# ----------------------------------------------------------------------

# Global variables (SHOULD begin with capital letter)

SELinux=""
EDAC=""
SCSMinvers="2.22.00"
SVRDMinvers="6.6-10"
KMPMinVers="8.40.04"
JavaMinVers="1.6"
SCSInstPath="/opt/fujitsu/ServerViewSuite/SCS"

PkgVers=$MYVERSION  # this is the version of the srvmagt package set
PkgDir=.            # default location where package files are
fOption=0           # 1: force an action as if confirmed with "yes"
NoStart=0           # 1: Suppress the automatic startup after installation
Voption=0           # 1: output verbose messages
instSVRAID=0        # 1: install SV-RAID rpm package if exists
JapLoc=0            # 1: Japanese Locality 
Quiet=0             # 1: No user interaction
InstSSM=1           # 0: do not install SSM
InstCIMProvs=0      # 1: install CIM providers

# ----------------------------------------------------------------------
# Default (English) messages. Can be overloaded by internationalized
# messages (e.g. for LANG="de_DE.UTF-8") if any.

MYNAME="ServerView Agents Linux"

Cancel="%s\n[C=Cancel]> "
Unknown="unknown"

BANNER="Installation script for %s v%s
Copyright 2018 FUJITSU LIMITED"

EulaAndThirdPartyLicenseReadme="
Software License Agreement for End Users and Third Party License, see
/usr/share/doc/packages/srvmagt-agents,
/usr/share/doc/packages/srvmagt-eecd and
/usr/share/srv-cimprovider/doc"

USAGE="Usage:
srvmagt [-f] [-n] [-R] [-h|--help] [-p path] [-v vers] [--ssm inst] [-V] \\
                [{install|start|restart|status|diag|stop|remove}]\n
        -h|--help
                display this information and exit
        --version
                display the script version and exit
        --ssm install | not-install  
                determines the installation of ServerView System Monitor
                (default: install)
        -f      force intended action against an issued warning/rejection
        -n      do not start the ServerView Agent processes after installation
        -R      install ServerView RAID package if available
        -p path indicates the path where the packages are located (default: .)
        -v vers indicates the version to handle (default: script version)
        -V      log more information on progress\n
        install install (upgrade) ServerView Agent packages
        start   start ServerView Agent processes
        restart stop followed by start
        status  show ServerView Agent status
        diag    collect ServerView Agent diagnostic information
        stop    stop ServerView Agent processes
        remove  remove ServerView Agent packages\n

You need root privileges. See also srvmagt(8).\n"

# Agent status map

map[0]="UNKNOWN"
map[1]="OK"
map[2]="DEGRADED"
map[3]="ERROR"
map[4]="FAILED"
map[5]="UNUSED"

# Status strings: MUST be two words!

Ok="OK status"
Error="General error"
ArgumentError="Argument error"
NotImplemented="Not implemented"
NotAuthorized="Not authorized"
MissingResource="Missing resources"
NotConfigured="Not configured"
NotRunning="Not running"
PartInstalled="Partially installed"
Undefined="Undefined status"

EmptyMessage="Empty message"
InvalidAction="Invalid option"
InvalidArgument="Invalid argument"
InvalidOpcode="Invalid opcode"
InvalidOpcodeExt="Invalid opcode extension"
NOTREACHED="Undefined logic flow"

Target="target system"
Reference="reference system"

Aborted="aborted"
Completed="completed"
DoesNotExist="missing"
Downgrade="downgrade"
Exists="exists"
Failed="failed!"
Install="install"
IsConfigured="requested"
IsLoaded="loaded"
IsNotConfigured="not requested"
IsNotLoaded="unloaded"
Missing="missing!"
None="none"
Nothing="retain"
NotInstalled="not installed"
Repair="repair"
Subagents="subagents"
Succeeded="succeeded!"
Unknown="unknown"
Upgrade="upgrade"

CimomMissing="no suitable CIMOM found"
CIMUnsupportedOS="OS is not supported"
CIMUnsupportedArch="32 bit architecture is not supported"
TooManyCIMOMs="More than one CIMOM found"


# Formatted messages

AllRPMsExist="All packages exist with version '%s'.\n"
BuildByCompiling="Building modules by compiling installed '%s' ...\n"
BuildByInstalling="Building modules by installing '%s' ...\n"
BuildCompleted="Building modules %s with status %d (%s %s).\n"
CannotAccessSVSCD="Cannot find ServerView directory on CD-ROM. No or wrong CD in drive?\n"
Checking="Checking ServerView Agents Linux configuration ...\n"
CopyPkgFromHost="Copy package '%s' from host '%s' via Secure Copy (scp) ...\n"
CopyPkgFromHostFailed="Copying package '%s' failed!\n"
DaemonStatus="Getting status of daemons ...\n"
Diagnose="Getting diagnostic material of ServerView Agents Linux ...\n"
DiagnoseCompleted="Getting diagnostic material %s with result %d (%s %s).\n"
DialogReturns="Dialog returned '%s'.\n"
EecdRunning="All eecd daemons are running.\n"
EnableSnmpdFailed="Enabling the snmpd init script via %s failed.\n"
FileSaved="File '%s' saved as '%s' \n"
FirewallReconfiguredPortsClosed="Reconfiguration of the Firewall: Ports '%s' closed for %s access. \n"
FirewallReconfiguredPortsOpened="Reconfiguration of the Firewall: Ports '%s' opened for %s access. \n"
ForceThisInstallation="Using Option -f you can perform this installation nevertheless.\n\n"
InstallationConflict="Installation conflicts with installed package: '%s'"
IncompletelyInstalled="%s is incompletely installed.\n"
InstallCompleted="Installing packages %s with status %d (%s %s).\n"
Installing="Installing ServerView Agents Linux '%s' from '%s' ...\n"
InstallingPkgRPM="Installing RPM '%s' from '%s' ...\n"
InstallingRPMs="Installing '%s' srvmagt packages from '%s' %s.\n"
InstallKmp="Installing KMP for %s.\n"
InstallKmpFailed="Installing of KMP for %s failed.\n"
Installfailed="Installing %s Package failed.\n"
InstallOnOS="You try to install or operate %s on OS '%s %s'.\n\n"
InsuffDiskSpace="Not enough disk space available\n"
InvalidIPAddr="Hostname or IP-Address '[%s' invalid, (cannot be reached).\n"
InvalidPkgDir="The package directory '%s' does not exist.\n"
InvalidPkgPath="Invalid path to '%s' package.\n"
InvalidPath="Invalid path argument '%s' (no leading . or /).\n"
InvalidUsage="Invalid action '%s'.\n%s\n"
IsAxxxSystem="This is a %s.\n"
LookupRPMfiles="Looking up srvmagt package files '%s' in '%s' ...\n"
LookupRPMs="Looking up installed srvmagt packages ...\n"
MainVersionSupported="Main Version is supported. Processing continues.\n"
MatchingKernelSource="Kernel source RPM '%s %s' matching OS '%s %s' %s.\n"
MissingInitScript="srvmagt init scripts %s %s.\n"
MissingKernelSource="There is probably no matching kernel source for kernel version %s installed. For details see /var/log/messages.\n\n"
MixedRPMs="There are mixed versions '%s %s' and '%s %s', %s.\n"
ModulesNotRequired="On '%s' compiled modules are not required .\n"
ModuleStatus="Getting status of kernel modules ...\n"
MustBeRoot="%s: You must be root to execute this program.\n"
NativeIPMI="Native IPMI modules"
NoKMP="No suitable KMP package found for '%s', version must be at least '%s'. This may result in a restricted functionality of ServerView Agents Linux.\n"
NoPkg="No suitable package found for '%s'. Version must be at least '%s'.\n"
NoRPM="Cannot find package '%s' to install.\n"
NoRPMfiles="No matching '%s' RPM files of '%s' in '%s'.\n"
NoSuchProcess="%27s: no such process found.\n"
NoSuchProgram="%27s: no such program file found.\n"
NotYetImplemented="Command '%s' is not implemented.\n"
OnTargetSystem="This is a 'target' system not equipped to build ServerView modules.\n"
OverruledOSvalidation="You have overruled OS validation with the -f option.\n"
OVMMissingRPMS="The required packages for %s are not installed. See readme file for details"
PathToModsBin="Please specify either the path to srvmagt-mods_bin package (absolute path definition), or the hostname/IP address of the reference system."
PerformAction="Perform %s of '%s' to '%s' ...\n"
RemovePkgFailed="Removing %s failed.\n"
RemoveCompleted="Removing %s with status %d (%s %s).\n"
Removing="Removing ServerView Agents Linux ...\n"
RemovingRPMs="Removing existing srvmagt packages %s.\n"
RemovingSCSfaild="Removing srvmagt-scs Package failed.\n"
RequiredRPMexists="The required package '%s' exists as '%s'.\n"
RequiredRPMmissing="The required package '%s' is not installed!\n"
Required32bitRPMmissing="The required 32bit package '%s' is not installed!\n"
RestartCompleted="Restarting %s with status %d (%s %s).\n"
Restarting="Restarting ServerView Agents Linux ...\n"
RPMcanNotBeInstalled="Package '%s' can not be installed: %s (might be OK).\n"
RPMnotInstalled="Package '%s' is not installed, %s.\n"
RPMsAlreadyInstalled="Packages of version '%s' are already installed.\n"
RPMsAvailable="Available packages:\n%s.\n"
RPMsOnReferenceSystem="RPMs required on a reference system:\n"
RPMstatus="Getting status of packages...\n"
Running="%27s: running (%s)\n"
RunningAgt="%27s: running (%s), status %s (%s)\n"
RunningEecd="%27s: running (%s), /dev/eecd[asj] %s\n"
RunningDaemon="%27s: running (%s), TCP %s %s\n"
RunningSnmpd="%27s: running (%s), UDP #161 %s, AgentX socket %s SNMP communication %s\n"
SELinuxIsEnabled="Driver Monitor functionality is not available with SELinux enabled.\n"
SnmpdNotConfigured="Native Net-SNMP agent not yet configured\n"
SomeFeaturesAreUnavailable="Some features of ServerView Agents Linux are not available.\n"
StartCompleted="Starting %s with status %d (%s %s).\n"
Starting="Starting ServerView Agents Linux ...\n"
StartingRPM="Starting '%s' %s.\n"
StartingRPMsWithTrace="Starting ServerView Agents Linux with trace enabled ...\n"
StartWithTraceCompleted="(Re)Starting with traces enabled %s with status %d (%s %s).\n"
Status="Getting status of ServerView Agents Linux ...\n"
StatusCompleted="Getting status %s with result %d (%s %s).\n"
StopCompleted="Stopping %s with status %d (%s %s).\n"
Stopping="Stopping ServerView Agents Linux ...\n"
StoppingRPMs="Stopping '%s' %s\n"
SubagentsRunning="All subagents are running.\n"
SVmodule="ServerView module"
SVRemoteConnectorRunning="Server View Remote Connector is running.\n"
SymLinkFailed="Symbolic link to package '%s/%s' failed\n"
TooOld="Installed version of '%s' is too old. Version must be at least '%s'.\n"
UnexpectedStatus="Unexpected status %d of '%s status'.\n"
UnsupportedOS="%s do not officially support OS '%s %s'.\n\n"
UnsupportedOSDisclaimer="Therefore, you install and operate %s on your own risk and responsibility and under exclusion of any liability and warranty by Fujitsu.\n\n"
Upgradefailed="Upgrading %s Package failed.\n"
UpgradingRPMs="Upgrading '%s' srvmagt packages from '%s' %s.\n"

SVaction="ServerView action script"
SVeecd="ServerView eecd daemon"
SNMPdaemon="SNMP daemon (snmpd)"
SVremCon="SVRemoteConnector"
SVRaiddaemon="RAID daemon (amDaemon)"

# Formatted messages: Relevant leading spaces!

         ServerModel="               Server model: %s\n"
            HostName="                   Hostname: %s (%s)\n"
     OperatingSystem="           Operating system: %s\n"
         LinuxKernel="               Linux kernel: %s on %s, SELinux %s, EDAC %s, %s\n"
      SVModuleKmpRPM="     ServerView kmp modules: %s\n"
         SVModuleRPM="         ServerView modules: %s\n"
           SVEecdRPM="            ServerView eecd: %s\n"
        SVAagentsRPM="  ServerView SNMP subagents: %s\n"
        SVCimProvRPM="    ServerView CIM provider: %s\n"
             SVUIRPM="      System Monitor Web UI: %s\n"  
SVRemoteConnectorRPM="ServerView Remote Connector: %s\n"
           SVRaidRPM="            ServerView Raid: %s\n"
      SNMPserviceRPM="               SNMP service: %s\n"

# ----------------------------------------------------------------------

# Log hello and determine the LANG

verbose "$(LC_ALL=C date +'%b %e %Y') %s %s %s %s: $*\n" $HOSTNAME $HASH $MYSELF $MYVERSION 2> /dev/null
logOnly "$HASH $MYSELF $MYVERSION: $*"

L=${LANG%_*}
[ "$L" = "en" ] || { 
	if [ -f ./srvmagt_$L.msg ]; then
		source ./srvmagt_$L.msg
	elif [ -f $SRVMAGT_MSG/srvmagt_$L.msg ]; then
		source $SRVMAGT_MSG/srvmagt_$L.msg
	else
		verbose "There is no message catalog %s, use English!\n" $SRVMAGT_MSG/srvmagt_$L.msg 2> /dev/null
	fi
}

# Set Japanese Locality if {LC_ALL|LANG} starts with "ja_" 
L=${LC_ALL%_*}
[ -z $L ] && L=${LANG%_*} 
if [ "$L" = "ja" ]; then
	JapLoc=1
else
	# On VMware {LC_ALL|LANG} is not correctly set, so test if date contains JST
	date | fgrep JST > /dev/null
	if [ $? = 0 ]; then
		JapLoc=1
	else  # last chance 
		[ -f /etc/sysconfig/keyboard ] &&
		fgrep KEYTABLE=\"jp  /etc/sysconfig/keyboard && JapLoc=1
	fi
fi

# Make sure that a decent shell and path is used and root privileges apply

PATH=/bin:/sbin:/usr/bin:/usr/sbin
[ -n "${BASH}" ] || {
	printf "$MustBeBash"
	exit 2
}

[ $(id -u) -eq 0 ] || {
	printf "$MustBeRoot" $MYSELF
	exit 4
}

# Please consider:
# The first parameter "---InternalFlock" is an internal one!
# It MUST NOT be used from the usual CLI of this script!
# If used -> behaviour of this script is undefined!

RunningUnderFlock="no"
if [ "$1" = "---InternalFlock" ]; then 
	RunningUnderFlock="yes"
	# we have to skip the first parameter "---InternalFlock"
	shift
fi


# Long options --help and --version are only recognized before short options
# --help and --version do nothing than print info, --ssm determines the installation of
# ServerView System Monitor.

options=""
while [ $# -gt 0 ]; do
	case "$1" in
	--help)
		printf "$BANNER\n\n" "$MYNAME" $PkgVers
		printf "$USAGE"
		exit 0 ;;
	--version)
		printf "$BANNER\n\n" "$MYNAME" $PkgVers
		exit 0 ;;
	--ssm)
		shift
		case "$1" in
			install)
				InstSSM=1 ;;
			not-install)
				InstSSM=0 ;;
			*)      # unkown action ignored
				printf "unkown action $1 for ssm ignored\n"
		esac
		;;
	*)
		options="$options $1" ;;
	esac
	shift
done
    
# Check short options
getSrvmagtOpts $options

if [ "$RunningUnderFlock" = "no" ]; then 
	printf "\n$BANNER\n\n" "$MYNAME" $MYVERSION
fi

getOSProperties || exit 151

onReferenceSystem

case $action in
	build)
		status=3
		display "$NotYetImplemented" "build" ;;

	install)
		printf "$EulaAndThirdPartyLicenseReadme\n\n"

		# Check if CIM providers are to be installed
		#

		if [ "$SV_ARCH" = "x86_64" ]; then
			case  $SV_SYSTEM  in
			suse)
				ver=${SV_NICKNAME#SLES}
				[ $ver -gt 10 ] && InstCIMProvs=1
				;;
			redhat)
				InstCIMProvs=1
				;;
			esac

			if [ $InstCIMProvs -eq 0 ]; then
				reason="$CIMUnsupportedOS"
			else
				# Check if a cimom is installed
				reason="$CimomMissing"
				InstCIMProvs=0
				[ -x "/usr/sbin/sfcbd" ] &&
					InstCIMProvs=1
				[ -x "/usr/sbin/cimserver" ] &&
					if [ $InstCIMProvs -eq 1 ]; then 
						InstCIMProvs=0
						reason="$TooManyCIMOMs"
					else
						InstCIMProvs=1
					fi
			fi
		else
			reason="$CIMUnsupportedArch"
		fi

		REPO_CREATED=0
		[ $Quiet -eq 1 ] && ASP_REPODIR="." && createRepo
		kmpsInstalled=0
		PLDPorDUPSupported || onReferenceSystem || {

			modsBinPkg="srvmagt-mods_bin-$PkgVers.$(uname -m).rpm"

			# skip asking for binary package if it already exists in the current directory
			[ -f "$PkgDir"/$modsBinPkg ] || {
				display "$MissingKernelSource" ${SV_RELEASE:1}
				[ $Quiet -eq 1 ] && {
					[ $REPO_CREATED -eq 1 ] && removeRepo
					exit 150
				}
				exit 5
				askDialog "$PathToModsBin" || exit 1
				display "$DialogReturns" $AnswerOfDialog

				if [ "${AnswerOfDialog:0:1}" = "/" ]; then
					remoteHost=0
				else
					remoteHost=1
				fi
				PkgDir=$(prepareTargetInstall "$PkgDir" $modsBinPkg $AnswerOfDialog $remoteHost) 
				[ $? -eq 0 ] || {
					display "$PkgDir\n"
					exit 5
				}

				[ $remoteHost -eq 1 ] && {
					display "$CopyPkgFromHost" $modsBinPkg $AnswerOfDialog

					copyBinPackage $modsBinPkg $AnswerOfDialog || {
						logOnly "$CopyPkgFromHostFailed" $binPkg
						exit 5
					}
				}
			}
		}
		[ $REPO_CREATED -eq 1 ] && removeRepo
		install
		status=$?
		display "$InstallCompleted" $(mapStatus $status) 
		ssmInstalled=1
		[ $status -eq 153 ] && ssmInstalled=0 && status=0
		[ $NoStart -eq 0 -a $status -eq 0 ] && {
			display "$Starting"
			startPkgs
			status=$?
 
			[ $JapLoc -eq 1 -a $status -eq 0 ] && {
				echo -e "Waiting for Inventory data ...\c" 
				count=300
				while [ $count -gt 0 ]; do
					[ -e  /etc/srvmagt/VersionView.sav ] && break;
					sleep 1;
					count=$(($count - 1))
					if [ $count -ge 100 ]; then
						echo -e "\b\b\b$count\c"
					elif [ $count -ge 10 ]; then
						echo -e "\b\b\b $count\c"
					else
						echo -e "\b\b\b  $count\c"
					fi  
				done
				if [ $count -eq 0 ]; then
					echo " giving up after 300 seconds!"
					display "Starting ServerView agents: VersionView.sav does not yet exist!\n"
					# status=1    # oder 7?
				else
					echo -e "\b\b\bdone ($count)"
				fi
			}
  
			display "$StartCompleted" $(mapStatus $status) 
			[ $ssmInstalled -eq 0 ]  && status=153 
			[ $kmpsInstalled -eq 2 ] && status=152

		} ;;

	check)
		# DISCUSSION:
		# display "$Checking"
		# checkConfiguration
		# status=$?
		# display "$CheckComplete" $(mapStatus $status) ;;
		status=3
		display "$NotYetImplemented" "diag" ;;

	start)
		if [ "$RunningUnderFlock" = "no" ]; then 
			# we are running unsafe -> restart this script by using flock

			export GrandFatherProcess=$$

			MaxWaitForLock=60	# in seconds
			display "Try to acquire lock from $GrandFatherProcess (wait $MaxWaitForLock seconds)\n"

			flock -o -w $MaxWaitForLock /var/log/srvmagt/lock.srvmagt-sbin $0 ---InternalFlock $options

			flockStatus=$?
			exit $flockStatus
		else
			# now, we have been restarted under flock (safe-mode)
			logOnly "lock acquired by $$ (ppid=$GrandFatherProcess)"
			display "$Starting"
			startPkgs
			status=$?
			display "$StartCompleted" $(mapStatus $status)
			logOnly "lock released by $$ (ppid=$GrandFatherProcess) status=$status"
		fi
		;;

	trace)
		if [ "$RunningUnderFlock" = "no" ]; then 
			# we are running unsafe -> restart this script by using flock

			export GrandFatherProcess=$$

			MaxWaitForLock=60	# in seconds
			display "Try to acquire lock from $GrandFatherProcess (wait $MaxWaitForLock seconds)\n"

			flock -o -w $MaxWaitForLock /var/log/srvmagt/lock.srvmagt-sbin $0 ---InternalFlock $options

			flockStatus=$?
			exit $flockStatus
		else
			# now, we have been restarted under flock (safe-mode)
			logOnly "lock acquired by $$ (ppid=$GrandFatherProcess)"
			display "$StartingRPMsWithTrace"
			startPkgs "trace"
			status=$?
			display "$StartWithTraceCompleted" $(mapStatus $status)
			logOnly "lock released by $$ (ppid=$GrandFatherProcess) status=$status"
		fi
		;;

	restart)
		if [ "$RunningUnderFlock" = "no" ]; then 
			# we are running unsafe -> restart this script by using flock

			export GrandFatherProcess=$$

			MaxWaitForLock=60	# in seconds
			display "Try to acquire lock from $GrandFatherProcess (wait $MaxWaitForLock seconds)\n"

			flock -o -w $MaxWaitForLock /var/log/srvmagt/lock.srvmagt-sbin $0 ---InternalFlock $options

			flockStatus=$?
			exit $flockStatus
		else
			# now, we have been restarted under flock (safe-mode)
			logOnly "lock acquired by $$ (ppid=$GrandFatherProcess)"
			display "$Restarting"
			stopPkgs
			startPkgs
			status=$?
			restartCIMOM 
			display "$RestartCompleted" $(mapStatus $status)
			logOnly "lock released by $$ (ppid=$GrandFatherProcess) status=$status"
		fi
		;;

	status)
		printf "$Status"
		getPkgStatus
		status=$?
		printf "$StatusCompleted" $(mapStatus $status) ;;

		diag)
		display "$Diagnose"
		collectDiagData
		status=$?
		display "$DiagnoseCompleted" $(mapStatus $status) ;;

	stop)
		if [ "$RunningUnderFlock" = "no" ]; then 
			# we are running unsafe -> restart this script by using flock

			export GrandFatherProcess=$$

			MaxWaitForLock=60	# in seconds
			display "Try to acquire lock from $GrandFatherProcess (wait $MaxWaitForLock seconds)\n"

			flock -o -w $MaxWaitForLock /var/log/srvmagt/lock.srvmagt-sbin $0 ---InternalFlock $options

			flockStatus=$?
			exit $flockStatus
		else
			# now, we have been restarted under flock (safe-mode)
			logOnly "lock acquired by $$ (ppid=$GrandFatherProcess)"
			display "$Stopping"
			stopPkgs
			status=$?
			display "$StopCompleted" $(mapStatus $status)
			logOnly "lock released by $$ (ppid=$GrandFatherProcess) status=$status"
		fi
		;;

	remove)
		display "$Removing"
		saveConf
		removePkgs
		status=$?
		# Remove ServerView Connector Service if possible and log it to /var/log/messages
		rpm -q --quiet ServerViewConnectorService && rpm -e ServerViewConnectorService 2>&1 | logger -t "srvmagt.sh"
		display "$RemoveCompleted" $(mapStatus $status) ;;


	*)  display "$InvalidUsage" "$action" "$USAGE"
		status=2 ;;
esac

# ----------------------------------------------------------------------
# Done
# ----------------------------------------------------------------------

exit $status

# EOF
